#pragma TextEncoding = "UTF-8"
#pragma rtGlobals=3		// Use modern global access method and strict wave access.
#pragma IgorVersion=9.0	// requires in-line multiple return syntax
#include "EphemerisTimeDelta" // Calculations of ephemeris time offset

// some astronomical constants
Static Constant sunradius = 696342 // km
Static Constant lunarmonth = 29.530587981 // days
Static Constant earthradius_equatorial = 6378137 // m
Static Constant earthradius_polar = 6356752.3142 // m
Static Constant earthradius = 6371.01 // km  
Static Constant earth_flattening = 0.00335281
Static Constant moonRadius = 1738 // km

// number of iterations in secant solvers
Static Constant iterations = 10

//___________________________________________________________________________________________________	
// secant method to try and find date of perihelion - not very accurate
Function sunSecantPerihelion(Variable date1, Variable targetLong)

	Variable date2 = date1 + (1/24)
	Variable Long1 = SunGeometricMeanLongitude(date1)
	
	Variable Long2 = SunGeometricMeanLongitude(date2)

	Long1-=targetLong
	Long2-=targetLong
	
	do
		Variable correction = (Long1 * (date1 - date2)) / (Long1-Long2)
		Variable dat = date1 - correction
		
		if (abs(correction) < (1/86400))
			return dat
		endif
	
		date2 = date1
		date1 = dat
		Long2 = Long1
		Long1 = SunGeometricMeanLongitude(dat)
		Long1-=targetLong
	while (1)
end
//___________________________________________________________________________________________________	
// from Meeus chapter 38, pass 0 for perihelion and 0.5 for aphelion - not very accurate
Function dateOfPerihelionAphelion(Variable JD, Variable Var)

	Variable year = str2num(JulianToDate(JD,3)[0,3])
	Variable y = year + ((JD - DateToJulian(year,01,01) - 0.5)/ 365.25)
	
	Variable k = 0.99997 * (y - 2000.01)
	
	k = floor(k) + Var
	
	Variable A1 = 238.41 + 132.788585 * k
	Variable A2 = 316.13 + 584.903153 * K
	Variable A3 = 346.20 + 450.380738 * k
	Variable A4 = 136.96 + 659.306737 * k
	Variable A5 = 249.52 + 326.653368 * k
	
	Variable correction
	if (var == 0.5)
		correction = -1.352 * sin(A1) + 0.061 * sin(A2) + 0.062 * sin(A3) + 0.029 * sin(A4) + 0.031 * sin(A5) 
	else
		correction = 1.278 * sin(A1) - 0.055 * sin(A2) - 0.091 * sin(A3) - 0.056 * sin(A4) - 0.045 * sin(A5) 
	endif	
	
	return 2451547.507 + 365.2596358*k + 0.0000000156*k^2 + correction 
end
//___________________________________________________________________________________________________	
// Julian Day, days since January 1, 4713 BC 12:00 UT 
Function JulianDay(datStr,timeStr,TimeZone)
	String datstr,timeStr // YYYYMMDD,hh:mm:ss
	Variable TimeZone
	
	String yearStr,monthStr,dayStr
	Splitstring/E="(\d{4})(\d{2})(\d{2})" datStr,yearStr,monthStr,dayStr
	
	Variable yearVar = str2num(yearStr)
	Variable monthVar = str2num(monthStr)
	Variable dayVar = str2num(dayStr)
	
	Variable hourVar,minuteVar,SecondVar
	sscanf timeStr, "%f:%f:%f", hourVar,minuteVar,SecondVar
	hourvar+= - 12 - timezone
		
	// days since January 1, 4713 BC
	return DateToJulian(yearVar,monthVar,dayVar)+(((hourVar*3600)+(minuteVar*60)+secondVar)/86400)
end
//___________________________________________________________________________________________________	
// Moon age in days	from a known new moon
Function getMoonAge(datStr,timeStr,TimeZone)
	String datStr, timeStr
	Variable TimeZone
	
	String yearStr,monthStr,dayStr
	Splitstring/E="(\d{4})(\d{2})(\d{2})" datStr,yearStr,monthStr,dayStr
	
	Variable yearVar = str2num(yearStr)
	Variable monthVar = str2num(monthStr)
	Variable dayVar = str2num(dayStr)
	
	Variable hourVar,minuteVar,SecondVar
	sscanf timeStr, "%f:%f:%f", hourVar,minuteVar,SecondVar
	hourvar-= timezone
	
	Variable currenttimeVar = date2secs(yearVar,monthVar,dayVar) + (hourVar*3600) + (minuteVar*60) + secondVar
	
	Variable moontimeVar = date2secs(2020,10,16) + (20*3600) + (31*60) // some known date of a new moon
	
	// The duration in days of a lunar cycle
	Variable lunardays =  29.530588861
		
	// Seconds in lunar cycle
	Variable lunarsecs = lunardays * 86400
	
	Variable totalSecs = currenttimevar - moontimeVar 
	
	Variable currentsecs = mod(totalsecs, lunarsecs)
	
	// Calculate the fraction of the moon cycle
	Variable currentfrac = (currentsecs /  lunarsecs)

	// Calculate days in current cycle (moon age)
	Variable currentdays = currentfrac * lunardays
	
	// deal with dates before 20000106
	if (currentDays<0)
		currentdays+=lunardays
		endif
	
	return currentdays
end
//___________________________________________________________________________________________________	
// Moon phase
Function/S getMoonPhase(MoonAge)
	Variable MoonAge

	String PhaseStr
	if ((0 <= MoonAge) & (MoonAge <1))
		PhaseStr="new"
	elseif ((1 <= MoonAge) & (MoonAge <6.38264692644))
		PhaseStr="waxing crescent"
	elseif ((6.38264692644 <= MoonAge) & (MoonAge <8.38264692644))
		PhaseStr="first quarter"
	elseif ((8.38264692644 <= MoonAge) & (MoonAge <13.76529385288))
		PhaseStr="waxing gibbous"
	elseif ((13.76529385288 <= MoonAge ) & (MoonAge<15.76529385288))
		PhaseStr="full"
	elseif ((15.76529385288 <= MoonAge) & (MoonAge <21.14794077932))
		PhaseStr="waning gibbous"
	elseif ((21.14794077932 <= MoonAge) & (MoonAge <23.14794077932))
		PhaseStr="last quarter"
	elseif ((23.14794077932 <= MoonAge) & (MoonAge <28.53058770576))
		PhaseStr="waning crescent"
	elseif ((28.53058770576 <= MoonAge) & (MoonAge <29.53058770576))
		PhaseStr="new"
	endif

	Return PhaseStr
end
//___________________________________________________________________________________________________	
// get a string for the shaddow length
Function/S shaddowLengthStr(Variable lenVar, String objStr)

	String rtnStr=""
	if (lenVar > 0)
		Sprintf rtnStr "%.3f m",lenVar
	else
		sprintf rtnStr "The %s is set",objStr
	endif

	Return rtnStr
end
//___________________________________________________________________________________________________	
// get JD from ephemeris time
Function JDE2JD(Variable JDE)
	return JDE - (deltaT(JDE)/86400) 
end
//___________________________________________________________________________________________________
// get decimal latitude as a degrees, hours, minutes string
Function/S DDlat2DMS(Variable dd[,Variable verb])
	
	String rtnStr,dirStr
	Variable d = trunc(dd)
	Variable m = trunc((dd - d) * 60)
	Variable s = (dd - d - m/60) * 3600
	
	if (ParamisDefault(verb))
		sprintf rtnStr,"%d° %d\' %.3f\"",d,abs(m),abs(s)
		return rtnStr
	endif

	if ((verb ==0)&&(dd>0)) // latitude north
		dirStr = "N"
	elseif ((verb ==0)&&(dd<0)) // latitude south
		dirStr = "S"
	elseif ((verb ==1)&&(dd>0)) // longitude east
		dirStr = "North"
	elseif ((verb ==1)&&(dd<0)) // longitude west
		dirStr = "South"
	else
		dirStr =""
	endif
	
	sprintf rtnStr,"%d° %d\' %.3f\" %s",abs(d),abs(m),abs(s),dirStr	
	
	return rtnStr
end
//___________________________________________________________________________________________________
// get decimal longitude as a degrees, hours, minutes string
Function/S DDlon2DMS(Variable dd[,Variable verb])
	
	String rtnStr,dirStr
	Variable d = trunc(dd)
	Variable m = trunc((dd - d) * 60)
	Variable s = (dd - d - m/60) * 3600
	
	if (ParamisDefault(verb))
		sprintf rtnStr,"%d° %d\' %.3f\"",d,abs(m),abs(s)
		return rtnStr
	endif

	if ((verb ==0)&&(dd>0)) // latitude north
		dirStr = "E"
	elseif ((verb ==0)&&(dd<0)) // latitude south
		dirStr = "W"
	elseif ((verb ==1)&&(dd>0)) // longitude east
		dirStr = "East"
	elseif ((verb ==1)&&(dd<0)) // longitude west
		dirStr = "West"
	else
		dirStr =""
	endif
	
	sprintf rtnStr,"%d° %d\' %.3f\" %s",abs(d),abs(m),abs(s),dirStr	
	
	return rtnStr
end
//___________________________________________________________________________________________________
// get right ascension/radians to hours minutes seconds string
Function/S RA2HMS(Variable RA)
	return Secs2Time((RA/(2*pi))*86400,5)
end
//___________________________________________________________________________________________________
// get degrees in hours minutes seconds string
Function/S DD2HMS(Variable DD)
	return Secs2Time(DD*240,5)
end
//___________________________________________________________________________________________________
// get a string indicating which half of the lunar month it is
Function/S WaxingWaning(Variable LunarAge)
	if (LunarAge < (29.5305555/2))
		Return "waxing"
	endif
	Return "waning"
end
//___________________________________________________________________________________________________
// Unix timestamp in seconds since 19700101
Function getUnixTime(datStr,timeStr,TimeZone)
	String datstr,timeStr // YYYYMMDD,hh:mm:ss
	Variable TimeZone
	
	String yearStr,monthStr,dayStr
	SplitString/E="(\d{4})(\d{2})(\d{2})" datStr,yearStr,monthStr,dayStr
	
	Variable yearVar = str2num(yearStr)
	Variable monthVar = str2num(monthStr)
	Variable dayVar = str2num(dayStr)
	
	Variable hourVar,minuteVar,SecondVar
	sscanf timeStr, "%f:%f:%f", hourVar,minuteVar,SecondVar
	hourvar-= timezone
	
	// seconds since 01/01/1970
	return Date2secs(yearVar,monthVar,dayVar)+((hourVar*3600)+(minuteVar*60)+secondVar) - Date2Secs(1970,1,1)
end
//___________________________________________________________________________________________________
// convert from igor time to julian 
Function DateTime2Julian(Variable DT)
	
	DT+=Date2Secs(-1,-1,-1) // Get UTC
	DT/=86400 // get days since start of igor epoch
	DT+=datetoJulian(1904,01,01) - 0.5	// add julian date from start of igor epoch

	Return DT 
end
//___________________________________________________________________________________________________
// get Heliocentric julian day - JD ajusted for speed of light 
Function HJD(Variable JD)
	Variable c = 299792458 // m/s
	Variable L0 = sunDistance(JD)
	
	return JD + (L0/c)/86400
end
//___________________________________________________________________________________________________
// Get julian in unix epoch
Function Julian2Unix(Variable JD)
	Return (JD - 2440587.5)*86400
end
//___________________________________________________________________________________________________
// Convert IgorTime to unix epoch
Function DateTime2Unix(Variable DT)
	Return DT - Date2Secs(1970,01,01)
end
//___________________________________________________________________________________________________
// Convert IgorTime to unix epoch
Function Unix2DateTime(Variable DT)
	Return DT + Date2Secs(1970,01,01)
end
//___________________________________________________________________________________________________
// date to Julian correctly
Function Date2Julian(Variable year,Variable month,Variable day)
	Return DateToJulian(year,month,day) - 0.5
end
//___________________________________________________________________________________________________
// return a time and date srtring form a julian date
Function/S Julian2DateTime(Variable JD[,Variable T,Variable D])
	
	// defaults 
	if (ParamIsDefault(T))
		T = 2 // 24hr time, no seconds
	endif
	if (ParamIsDefault(D))
		D =2 // e.g. dayStr monthStr DD, YYYY
	endif
	
	return Secs2Time((JD-0.5)*86400,T)+", "+JulianToDate(JD+0.5,D)
end
//___________________________________________________________________________________________________
// return a time string from a julian date
Function/S Julian2Time(Variable JD[,Variable T])
	
	// defaults 
	if (ParamIsDefault(T))
		T = 2 // 24hr time, no seconds
	endif

	return Secs2Time((JD-0.5)*86400, T)
end
//___________________________________________________________________________________________________
// convert from degrees to radians
Function rad(Variable num)
	Return num * (pi/180)
end
//___________________________________________________________________________________________________
// Convert from radians to degrees
Function deg(Variable num)
	return num * (180/pi)
end
//___________________________________________________________________________________________________	
// Get Julian Ephemeris Day - see EphemerisTimeDelta
Function JDE(Variable JD)
	return JD + (deltaT(JD)/86400) 
end
//___________________________________________________________________________________________________
// modified julian date
Function MJD(Variable JD)
	Return JD - 2400000.5
end
//___________________________________________________________________________________________________
// Lilian date, Count of days of the Gregorian calendar
Function LilianDate(Variable JD)
	return floor(JD - 2299159.5)
end
//___________________________________________________________________________________________________
// Rata Die, Count of days of the Common Era
Function RataDie(Variable JD)
	return floor(JD - 1721424.5)
end
//___________________________________________________________________________________________________
// get day number from Julian
Function Julian2DayNumber(Variable JD)
	Return floor(mod(JD - DateToJulian(str2num(JulianToDate(JD,4)[0,3]),01,01) + 0.5,365.25)) + 1
end
//___________________________________________________________________________________________________
// get number of days in the year
Function Julian2Days(Variable JD)
	Variable year = str2num(JulianToDate(JD,3)[0,3])
	Return mod(datetoJulian(year,12,31) - datetoJulian(year,01,01),365.25) + 1
end
//___________________________________________________________________________________________________
// get day number from Datetime
Function DateTime2DayNumber(Variable DT)
	 variable yyyy, mm, dd
    sscanf Secs2Date(DT,-2), "%g-%g-%g", yyyy, mm, dd

    return dateToJulian(yyyy, mm, dd)-dateToJulian(yyyy, 1, 1)+1
end
//___________________________________________________________________________________________________
// calculate length of shadow in meters, input altitude (alpha) in radians
Function ShadowLength(Variable a)
	if (a>0)
		return 1/tan(a)
	endif
	return 0 
end
//___________________________________________________________________________________________________
// Calculate the semidiameter of the Moon in arc seconds for the given julian day.
Function getLunarSemidiameter(Variable JD)

	Variable So = 358473400
	Variable dist = moonDistance(JD)

	return So / dist
end
//___________________________________________________________________________________________________
// Calculate the semidiameter of the Sun in arc seconds for the given julian day.
Function getSolarSemidiameter(Variable JD)

	Variable So = 959.63
	Variable dist = sunDistance(JD)

	return So / dist
end
//___________________________________________________________________________________________________
// return a string than indicates if a radial velocity is towards (negative) or away (positive)	
Function/S directionStr(Variable v)
	if (v<0)
		return "receding"
	endif
	return "approaching"
end
//___________________________________________________________________________________________________
// return the offset between apparent and mean solar time in radians
Function EquationOfTime(variable JD)
	
	Variable L0 = rad(sunGeometricMeanLongitude(JD))
	Variable M = rad(sunMeanAnomaly(JD))
	Variable Ob = rad(trueObliquityOfEcliptic(JD))
	Variable y = tan(Ob/2)*tan(Ob/2)
	Variable e = earthEccentricity(JD)
	
	return (y*sin(2*L0)-(2*e*sin(M))+(4*e*y*sin(M)*cos(2*L0))-(0.5*y*y*sin(4*L0))-(1.25*e*e*sin(2*M)))
end
//___________________________________________________________________________________________________
// return true solar time in radians
Function TrueSolarTime(Variable JD, Variable long)
	
	Variable ET = EquationOfTime(JD)
	Variable day = JD-(floor(JD)-0.5)

	Return ((2*pi)*day) + ET + rad(long)
end
//___________________________________________________________________________________________________
// Observer angle
Function ObserverAngle(Variable altitude) 
	
	// in degrees
	return -2.076 * sqrt(altitude) / 60
end
//___________________________________________________________________________________________________	
// This is a library of calculations for calculating astronomical positions.
// Most algorithms are based on Jean Meeus book _Astronomical Algorithms_

// Returns the value of omega from Chapter 25 of Jean Meeus.
Function meeusOmega(Variable JD)  
	
	Variable T = meeusT(JD)
	
	return 125.04 - 1934.136*T
end
//___________________________________________________________________________________________________
// Returns the apparent longitude of the sun (lambda) (low accuracy)
// from Meeus, Chapter 25
Function sunApparentLongitude(Variable JD) 
	
	Variable T = meeusT(JD)
	Variable omega = meeusOmega(JD)
	
	return sunTrueLongitude(JD) - 0.00569  - 0.00478*sin(omega)
end
//___________________________________________________________________________________________________
// Returns geometric mean longitude L0 of the sun, referred to the mean equinox
// of the date.  Equation 25.2 in Jean Meeus.
Function sunGeometricMeanLongitude(Variable JD)  
	
	Variable T = meeusT(JD)
	
	return mod(280.46646 + 36000.76983*T + 0.0003032*T^2,360)
end
//___________________________________________________________________________________________________
// Returns true longitude (sunSymbol) of the sun for a specified date.
// From Chapter 25 in Jean Meeus.
Function sunTrueLongitude(Variable JD)  

   return sunGeometricMeanLongitude(JD) + sunCenter(JD)
end
//___________________________________________________________________________________________________
// Returns the mean anomaly M of the sun for a given date.
// Equation 25.3 in Jean Meeus.
Function sunMeanAnomaly(Variable JD)  

	Variable T = meeusT(JD)
	
	return 357.52911 + 35999.05029*T - 0.0001537*T^2
end
//___________________________________________________________________________________________________
// Returns the true anomaly nu of the sun for a given date.
// From Chapter 25 of Meeus.
Function sunTrueAnomaly(Variable JD)  

	Variable T = meeusT(JD)
	
	return sunMeanAnomaly(JD) + sunCenter(JD)
end
//___________________________________________________________________________________________________
// Equation for center of the Sun C from Chapter 25 of Meeus.
Function sunCenter(Variable JD) 
	
	Variable T = meeusT(JD)
	Variable M = rad(sunMeanAnomaly(JD))
	
	return (1.914602 - 0.004817*T - 0.000014*T^2) * sin(M) + \
	(0.019993 - 0.000101*T) * sin(2*M) + \
	0.000289 * sin(3*M)
end
//___________________________________________________________________________________________________
// Returns the eccentricity of the earths orbit for a given date.
// Equation 25.4 in Meeus.
Function earthEccentricity(Variable JD) 
	
	Variable T = meeusT(JD)
	
	return 0.016708634 - 0.000042037*T - 0.0000001267*T^2
end
//___________________________________________________________________________________________________
// Returns the longitude of the perihelion of the orbit for the eccentric
// orbit calculated above.  This appears in Meeus before eq. 23.2. and is
// called pi in the text.
Function longitudeOfEarthPerihelion(Variable JD) 

	Variable T = meeusT(JD)
	
	return 102.93735 + 1.71946*T + 0.00046*T^2 
end
//___________________________________________________________________________________________________
// Distance between center of Sun and Earth
Function sunDistance(Variable JD) 
	
	// in meters
	return 149597870700 * sunRadialVector(JD)
end
//___________________________________________________________________________________________________
// Sun radial vector, Meeus eq 25.2
Function sunRadialVector(variable JD)	
	
	Variable eccentricity = earthEccentricity(JD)
	Variable trueAnomaly = rad(sunTrueAnomaly(JD))
	
	return 1.000001018 * ( 1 - eccentricity^2 ) / (1 + eccentricity * cos(trueAnomaly))
end
//___________________________________________________________________________________________________
// Calculates (numerically) the earth's velocity towards the sun, in m/s.
Function sunRadialVelocity(Variable JD) 
	
	Variable halfStep = (0.5 / 86400) // s
	Variable d1 = JD - halfStep
	Variable d2 = JD + halfStep
	
	return (sunDistance(d1) - sunDistance(d2)) / ((halfStep * 2)*86400)
end
//___________________________________________________________________________________________________
// Calculates the apparent sun radius angle, corrected for distance, but not
// corrected for refraction.  It gives a true arcsin geometric interpretation.
Function sunRadiusAngle(Variable JD)
	
	return asin(sunradius / (sunradius + sunDistance(JD)))
end
//___________________________________________________________________________________________________
// This returns the number of Julian centuries since JDE 2451545 as a 
// dimensionless number.
// Equation 25.1 in Jean Meeus
Function meeusT(Variable JD)  

	return (JDE(JD) - 2451545) / 36525
end
//___________________________________________________________________________________________________
// Mean obliquity of the ecliptic epsilon0 from Jean Meeus, eq. 22.3.  This 
// is a high accuracy version of the Function that is valid for a period
// of 10000 years before or after J2000.0
// It is NOT corrected for nutation.
// In Meeus, this symbol is referred to as epsilon_0
Function meanObliquityOfEcliptic(Variable JD) 
	
	Variable T = meeusT(JD) // julian centuries
	Variable U = T/100 // 10000 julian jears since J2000
	
	// in degrees
	return 23.43929 - 1.300258*U - 1.55*U^2 + 1999.25*U^3 - 51.38*U^4 - \
	249.67*U^5 - 39.05*U^6 + 7.12*U^7 + 27.87*U^8 + 5.79*U^9 +  2.45*U^10 
end
//___________________________________________________________________________________________________
// True obliquity of the ecliptic corrected for nutation.  This is called
// epsilon in Meeus.
Function trueObliquityOfEcliptic(Variable JD) 
	
	Variable epsilon0 = meanObliquityOfEcliptic(JD)
	Variable deltapsi, deltaepsilon
	[deltapsi, deltaepsilon] = highAccuracyNutation(JD)
	
	// in degrees
	return epsilon0 + deltaepsilon
end
//___________________________________________________________________________________________________
// Mean obliquity of the ecliptic epsilon0 from Jean Meeus, eq. 22.2.  This 
// is a low accuracy version of the Function that is only valid for a period
// of 1000 years before or after J2000.0
// In Meeus, this symbol is referred to as epsilon_0
Function narrowValidityMeanObliquityOfEcliptic(Variable JD) 
	
	Variable T = meeusT(JD)
	
	// in degrees	
	return 23.43929 + -0.01300417*T - (1.638889e-7)*T^2 + (5.036111e-7)*T^3 
end
//___________________________________________________________________________________________________
// Returns the apparent right ascension and declination of the sun for a
// given date.  It implements the corrected versions of equation 25.6
// and 25.7 in Meeus.
// Returns (ra, decl)
Function [Variable  ra, Variable  decl] sunApparentRADecl(Variable JD) 
	
	Variable epsilon = trueObliquityOfEcliptic(JD)
	Variable epsilonprime = rad(epsilon + 0.00256*cos(meeusOmega(JD)))
	Variable sunLong = rad(sunApparentLongitude(JD))
	Variable ssl = sin(sunLong)
	ra = rad(360) + atan2(cos(epsilonprime)*ssl, cos(sunLong))
	decl = asin(sin(epsilonprime)*ssl)

	// in radians
	return [ra, decl]
end
//___________________________________________________________________________________________________
// Calculate mean Sidereal time theta_0 at Greenwich for any instant as an
// angle.
// Equation 12.4 in Meeus.
Function meanGreenwichSiderealAngle(Variable JD) 
	
	Variable T = meeusT(JD)
	
	// in degrees
	return mod((280.46061837 + 360.98564736629 * (JD - 2451545) +  (0.000387933 * T^2) - (T^3 / 38710000)) + 360,360)
end
//___________________________________________________________________________________________________
// Calculate apparent Sidereal time as an angle at Greenwich for any instant.
// From Chapter 12 of Meeus.
Function apparentGreenwichSiderealAngle(Variable JD) 
	
	[Variable deltapsi,Variable deltaEpsilon] = highAccuracyNutation(JD)
	Variable trueObliquity = trueObliquityOfEcliptic(JD)
	Variable correction = deltapsi * cos(trueObliquity)
	
	// in degrees
	return meanGreenwichSiderealAngle(JD) + deg(correction)
end
//___________________________________________________________________________________________________
// Calculate apparent local sidereal time as an angle.
Function apparentLocalSiderealAngle(Variable JD,Variable longitude) 
	
	Variable angle = apparentGreenwichSiderealAngle(JD)
	
	// in degrees
	return angle + longitude
end
//___________________________________________________________________________________________________
// Calculate apparent mean sidereal time as an angle.
Function meanLocalSiderealAngle(Variable JD,Variable longitude) 
	
	Variable angle = meanGreenwichSiderealAngle(JD)
	
	// in degrees
	return angle + longitude
end
//___________________________________________________________________________________________________
// Calculate the hour angle H of a body with right ascension ra.
// See chapter 13 in Meeus.
Function hourAngle(Variable JD,Variable  longitude,Variable  ra) 
	
	// in degrees
	return apparentLocalSiderealAngle(JD, longitude) - deg(ra)
end
//___________________________________________________________________________________________________
// Returns the parallactic angle q for a given body from a point (lat, long)
// on the earth of a body at right ascension ra
// and declination decl.
// See equation 14.1 in Meeus.
Function parallacticAngle(JD, lat, long, ra, decl) 
	Variable JD, lat, long, ra, decl
	
	Variable H = rad(hourAngle(JD, long, ra))
	
	return atan2(sin(H), tan(rad(lat)) * cos(decl) - sin(decl) * cos(H))
end
//___________________________________________________________________________________________________
// Calculate the approximate date of the equinoxes and solstices
// (to medium accuracy.)  The value should be correct within a minute;
// see Table 27.D in Meeus.
// This is from chapter 27 of Meeus.
// This Function is called by the equinox and solstice methods below.
Function equinoxdate(Variable JDE0) 
	
	Variable T = (JDE0 - 2451545.0) / 36525
	Variable W = rad(35999.373 * T - 2.47)
	Variable deltaLambda = 1 + 0.0334 * cos(W) + 0.0007 * cos(2*W)
	
	// These values are from Table 27.C
	Variable S = 485*cos(rad(324.96 + 1934.136*T)) + \
		203*cos(rad(337.23 + 32964.467*T)) + \
		199*cos(rad(342.08 + 20.186*T)) + \
		182*cos(rad(27.85 + 445267.112*T)) + \
		156*cos(rad(73.14 + 45036.886*T)) + \
		136*cos(rad(171.52 + 22518.443*T)) + \
		77*cos(rad(222.54 + 65928.934*T)) + \
		74*cos(rad(296.72 + 3034.906*T)) + \
		70*cos(rad(243.58 + 9037.513*T)) + \
		58*cos(rad(119.81 + 33718.147*T)) + \
		52*cos(rad(297.17 +  150.678*T)) + \
		50*cos(rad(21.02 + 2281.226*T)) + \
		45*cos(rad(247.54 + 29929.562*T)) + \
		44*cos(rad(325.15 + 31555.956*T)) + \
		29*cos(rad(60.93 + 4443.417*T)) + \
		18*cos(rad(155.12 + 67555.328*T)) + \
		17*cos(rad(288.79 + 4562.452*T)) + \
		16*cos(rad(198.04 + 62894.029*T)) + \
		14*cos(rad(199.76 + 31436.921*T)) + \
		12*cos(rad(95.39 + 14577.848*T)) + \
		12*cos(rad(287.11 + 31931.756*T)) + \
		12*cos(rad(320.81 + 34777.259*T)) + \
		9*cos(rad(227.73 + 1222.114*T)) + \
		8*cos(rad(15.45 + 16859.074*T))
	
	Variable ss = JDE0 + (0.00001*S)/deltaLambda
	
	return JDE(ss)
end
//___________________________________________________________________________________________________
// Spring Equinox for the years +1000 to +3000, Table 27.B in Meeus.
// The year passed in must be an integer!
Function springEquinox(Variable year) 
	
	Variable Y = (year - 2000)/1000
	Variable JDE0 = 2451623.80984 + 365242.37404*Y + 0.05169*Y^2 - 0.00411*Y^3 -  0.00057*Y^4
	
	return equinoxdate(JDE0)
end
//___________________________________________________________________________________________________
// Summer Solstice for the years +1000 to +3000, Table 27.B in Meeus.
// The year passed in must be an integer!
Function summerSolstice(Variable year) 
	
	Variable Y = (year - 2000)/1000
	Variable JDE0 = 2451716.56767 + 365241.62603*Y + 0.00325*Y^2 + 0.00888*Y^3 -  0.00030*Y^4

	return equinoxdate(JDE0)
end
//___________________________________________________________________________________________________
// Autumn Equinox for the years +1000 to +3000, Table 27.B in Meeus.
// The year passed in must be an integer!
Function autumnEquinox(Variable year) 
	
	Variable Y = (year - 2000)/1000
	Variable JDE0 = 2451810.21715 + 365242.01767*Y - 0.11575*Y^2 + 0.00337*Y^3 +  0.00078*Y^4
	
	return equinoxdate(JDE0)
end
//___________________________________________________________________________________________________
// Winter Solstice for the years +1000 to +3000, Table 27.B in Meeus.
// The year passed in must be an integer!
 Function winterSolstice(Variable year) 
	
	Variable Y = (year - 2000)/1000
	Variable JDE0 = 2451900.05952 + 365242.74049*Y - 0.06223*Y^2 - 0.00823*Y^3 +  0.00032*Y^4
	
	return equinoxdate(JDE0)
end
//___________________________________________________________________________________________________
// Calculate Nutation in longitude (delta psi) and
// obliquity (delta epsilon) to low accuracy.  
// Returns (deltapsi, deltaepsilon)
// From chapter 22 in Meeus.
Function [Variable  deltapsi, Variable  deltaepsilon] lowAccuracyNutation(Variable JD) 
	
	Variable T = meeusT(JD)
	Variable omega = 125.04452 - 1934.136261*T + 0.0020708*T^2 + T^3/450000
	
	// Mean longitude of the sun
	Variable L = 280.4665 + 36000.7698*T
	
	// Mean longitude of the moon
	Variable Lmoon = 218.3165 + 481267.8813*T 
	
	deltapsi = 0.004777778*sin(omega) - 0.0003666667*sin(2*L) - 0.00006388889*sin(2*Lmoon) +  0.00005833333*sin(2*omega)  
	
	deltaepsilon = 0.002555556*cos(omega) + 0.0001583333*cos(2*L) + 0.00002777778*cos(2*Lmoon) -  0.00002777778*cos(2*omega) 
	
	return [deltapsi, deltaepsilon]
end
//___________________________________________________________________________________________________
// Calculate Nutation in longitude (delta psi) and
// obliquity (delta epsilon) to high precision.
// Returns (deltapsi, deltaepsilon)
// From chapter 22 in Meeus.
Function [Variable  deltapsi, Variable  deltaepsilon] highAccuracyNutation(Variable JD) 
	
	Variable T = meeusT(JD)
	
	// Mean elongation of the moon from the sun:
	Variable  D = rad(mod(297.85036 + 445267.111480*T - 0.0019142*T^2 + T^3/189474,360)) 
	
	// Mean anomaly of the Sun:
	Variable  M = rad(mod(357.52772 + 35999.050340*T - 0.0001603*T^2 - T^3/300000,360))
	
	// Mean anomaly of the Moon:
	Variable MP = rad(mod(134.96298 + 477198.867398*T + 0.0086972*T^2 + T^3/56250,360))
	
	// Moon's argument of Latitude:
	Variable F = rad(mod(93.27191 + 483202.017538*T - 0.0036825*T^2 + T^3/327270,360))
	
	// Longitude of the ascending node of the moon's mean orbit on the ecliptic
	// measured from the mean equinox of the date.
	Variable Omega = rad(mod(125.04452 - 1934.136261*T + 0.0020708*T^2 + T^3/450000,360)) 
	
	// These lines generated by iau1980.frink and pasted in here.
	deltapsi = (0.0001/3600)* ( + (-171996 + -174.2*T)*sin( + Omega) + (-13187 + -1.6*T)*sin( -2*D + 2*F + 2*Omega) + \
	(-2274 -0.2*T)*sin( + 2*F + 2*Omega) + (2062 + 0.2*T)*sin( + 2*Omega) + (1426 + -3.4*T)*sin( + M) + (712 + 0.1*T)*sin( + MP) + \
	(-517 + 1.2*T)*sin( -2*D + M + 2*F + 2*Omega) + (-386 + -0.4*T)*sin( + 2*F + Omega) + (-301)*sin( + MP + 2*F + 2*Omega) + \
	(217 -0.5*T)*sin( -2*D -1*M + 2*F + 2*Omega) + (-158)*sin( -2*D + MP) + (129 + 0.1  *T)*sin( -2*D + 2*F + Omega) + \
	(123)*sin( -1*MP + 2*F + 2*Omega) + (63)*sin( + 2*D) + (63 + 0.1*T)*sin( + MP + Omega) + (-59)*sin( + 2*D -1*MP + 2*F + 2*Omega) + \
	(-58 -0.1*T)*sin( -1*MP + Omega) + (-51)*sin( + MP + 2*F + Omega) + (48)*sin( -2*D + 2*MP) + (46)*sin( -2*MP + 2*F + Omega) + \
	(-38)*sin(2*D + 2*F + 2*Omega) + (-31)*sin(2*MP + 2*F + 2*Omega) + (29)*sin(2*MP) + (29)*sin(-2*D + MP + 2*F + 2*Omega) + \
	(26)*sin(2*F) + (-22)*sin(-2*D + 2*F) + (21)*sin(-1*MP + 2*F + Omega) + (17 + -0.1*T)*sin(2*M) + (16)*sin(2*D -1*MP + Omega) + \
	(-16 + 0.1*T)*sin( -2*D + 2*M + 2*F + 2*Omega) + (-15)*sin( + M + Omega) + (-13)*sin( -2*D + MP + Omega) + (-12)*sin( -1*M + Omega) + \
	(11)*sin( + 2*MP -2*F) + (-10)*sin( + 2*D -1*MP + 2*F + Omega) + (-8)*sin( + 2*D + MP + 2*F + 2*Omega) + (7)*sin( + M + 2*F + 2*Omega) + \
	(-7)*sin( -2*D + M + MP) + (-7)*sin( -1*M + 2*F + 2*Omega) + (-8)*sin( + 2*D + 2*F + Omega) + (6)*sin( + 2*D + MP) + \
	(6)*sin( -2*D + 2*MP + 2*F + 2*Omega) + (6)*sin( -2*D + MP + 2*F + Omega) + (-6)*sin( + 2*D -2*MP + Omega) + (-6)*sin( + 2*D + Omega) + \
	(5)*sin( -1*M + MP) + (-5)*sin( -2*D -1*M + 2*F + Omega) + (-5)*sin( -2*D + Omega) + (-5)*sin( + 2*MP + 2*F + Omega) + \
	(4)*sin( -2*D + 2*MP + Omega) + (4)*sin( -2*D + M + 2*F + Omega) + (4)*sin( + MP -2*F) + (-4)*sin( -1*D + MP) + (-4)*sin( -2*D + M) + \
	(-4)*sin( + D) + (3)*sin( + MP + 2*F) + (-3)*sin( -2*MP + 2*F + 2*Omega) + (-3)*sin( -1*D -1*M + MP) + (-3)*sin( + M + MP) + \
	(-3)*sin( -1*M + MP + 2*F + 2*Omega) + (-3)*sin( + 2*D -1*M -1*MP + 2*F + 2*Omega) + (-3)*sin( + 3*MP + 2*F + 2*Omega) + \
	(-3)*sin( + 2*D -1*M + 2*F + 2*Omega))
	
	deltaepsilon = (0.0001/3600) * ( + (92025 + 8.9*T)*cos( + Omega) + (5736 + -3.1*T)*cos( -2*D + 2*F + 2*Omega) + \
	(977 + -0.5*T)*cos( + 2*F + 2*Omega) + (-895 + 0.5*T)*cos( + 2*Omega) + (54 + -0.1*T)*cos( + M) + (-7)*cos( + MP) + \
	(224 + -0.6*T)*cos( -2*D + M + 2*F + 2*Omega) + (200)*cos( + 2*F + Omega) + (129 + -0.1*T)*cos( + MP + 2*F + 2*Omega) + \
	(-95 + 0.3*T)*cos( -2*D -1*M + 2*F + 2*Omega) + (-70)*cos( -2*D + 2*F + Omega) + (-53)*cos( -1*MP + 2*F + 2*Omega) + \
	(-33)*cos( + MP + Omega) + (26)*cos( + 2*D -1*MP + 2*F + 2*Omega) + (32)*cos( -1*MP + Omega) + (27)*cos( + MP + 2*F + Omega) + \
	(-24)*cos( -2*MP + 2*F + Omega) + (16)*cos( + 2*D + 2*F + 2*Omega) + (13)*cos( + 2*MP + 2*F + 2*Omega) + (-12)*cos( -2*D + MP + 2*F + 2*Omega) + \
	(-10)*cos( -1*MP + 2*F + Omega) + (-8)*cos( + 2*D -1*MP + Omega) + (7)*cos( -2*D + 2*M + 2*F + 2*Omega) + (9)*cos( + M + Omega) + \
	(7)*cos( -2*D + MP + Omega) + (6)*cos( -1*M + Omega) + (5)*cos( + 2*D -1*MP + 2*F + Omega) + (3)*cos( + 2*D + MP + 2*F + 2*Omega) + \
	(-3)*cos( + M + 2*F + 2*Omega) + (3)*cos( -1*M + 2*F + 2*Omega) + (3)*cos( + 2*D + 2*F + Omega) + (-3)*cos( -2*D + 2*MP + 2*F + 2*Omega) + \
	(-3)*cos( -2*D + MP + 2*F + Omega) + (3)*cos( + 2*D -2*MP + Omega) + (3)*cos( + 2*D + Omega) + (3)*cos( -2*D -1*M + 2*F + Omega) + \
	(3)*cos( -2*D + Omega) + (3)*cos( + 2*MP + 2*F + Omega))
	
	return [deltapsi, deltaepsilon]
end
//___________________________________________________________________________________________________
// Calculate the changes in right ascension and declinaton due to nutation.
//    The calculations above calculate nutation in ecliptical latitude and
//    longitude, but this recasts those corrections to right ascension and
//    declination.  This is equation 23.1 in Meeus.
//
//    arguments:
//       date:  The date to be calculated
//
//    returns (deltaRA, deltaDecl) , the corrections to add to right ascension
//                                   and declination respectively.
//
Function [Variable deltaRA, Variable deltaDecl] highAccuracyNutationInRADecl(Variable JD, Variable ra, Variable decl) 
	
	Variable epsilon = meanObliquityOfEcliptic(JD)
	[Variable deltapsi,Variable deltaepsilon] = highAccuracyNutation(JD)
	Variable ce = cos(epsilon)
	Variable se = sin(epsilon)
	Variable sa = sin(ra)
	Variable ca = cos(ra)
	Variable td = tan(decl)
	
	deltaRA = (ce + se*sa*td)*deltapsi - (ca*td)*deltaepsilon
	deltaDecl = (se*ca)*deltapsi + sa*deltaepsilon
	
	return [deltaRA, deltaDecl]
end
//___________________________________________________________________________________________________
// Calculate the effect of annual aberration.  This is Meeus 23.2 and 23.3
//
//    TODO:  Possibly implement the Ron-Vondrak equations for aberration, see
//    Meeus p. 153
//
//    returns:
//       (deltaRA, deltaDecl) : the corrections to be added to right ascension
//                              and declination.
//
Function [Variable deltaRA, Variable deltaDecl] annualAberration(Variable JD, Variable ra, Variable decl)  

	Variable eccentricity = earthEccentricity(JD)
	Variable perihelionLong = longitudeOfEarthPerihelion(JD)
	Variable k = rad(0.0056932)  // Constant of aberration, degrees
	Variable sunLong = sunTrueLongitude(JD)
	Variable epsilon = meanObliquityOfEcliptic(JD)
	
	// Equations 23.3
	deltaRA = -k*(cos(ra)*cos(sunLong)*cos(epsilon) + sin(ra)*sin(sunLong)) / cos(decl) + eccentricity*k*(cos(ra)*cos(perihelionLong)*cos(epsilon) + sin(ra)*sin(perihelionLong)) / cos(decl)
	deltaDecl = -k*(cos(sunLong)*cos(epsilon)*(tan(epsilon)*cos(decl) - sin(ra)*sin(decl)) + cos(ra)*sin(decl)*sin(sunLong)) + eccentricity*k*(cos(perihelionLong)*cos(epsilon)*(tan(epsilon)*cos(decl) - sin(ra)*sin(decl)) + cos(ra)*sin(decl)*sin(perihelionLong))
	
	return [deltaRA, deltaDecl]
end
//___________________________________________________________________________________________________
// Calculate the right ascension and declination (equatorial coordinates)
// of an object given ecliptical coordinates.  This is an implementation of
// eq. 13.3 and 13.4 in Meeus.
//
// arguments:
//  (lambda, beta, epsilon)
//
//   where:
//     lambda:  ecliptic longitude
//     beta:    ecliptic latitude (beeta!)
//     epsilon: obliquity of the ecliptic
//
// returns:
//  (ra, decl)
Function [Variable alpha, Variable delta] eclipticalToRADecl(Variable lambda,Variable  beeta,Variable  epsilon) 
   
	alpha = atan2(sin(lambda)*cos(epsilon) - tan(beeta)*sin(epsilon), cos(lambda))
	delta = asin(sin(beeta)*cos(epsilon) + cos(beeta)*sin(epsilon)*sin(lambda))
	
	return [alpha, delta]
end
//___________________________________________________________________________________________________
// Calculate the ecliptical coordinates of an object given right ascension
// and declination (equatorial coordinates).
// eq. 13.1 and 13.2 in Meeus.
//
// arguments:
//  (ra, decl, epsilon)
//
//   where:
//     ra:   right ascension (can be angle or time)
//     decl: declination
//     epsilon: obliquity of the ecliptic
//
// returns:
//  (lambda, beta)
//
//  where:
//     lambda:  ecliptic longitude
//     beta:    ecliptic latitude
Function [Variable lambda, Variable beeta] raDeclToEcliptical(Variable ra,Variable  decl,Variable  epsilon) 

	lambda = atan2(sin(ra)*cos(epsilon) + tan(decl)*sin(epsilon), cos(ra))
	beeta = asin(sin(decl)*cos(epsilon) - cos(decl)*sin(epsilon)*sin(ra))
	
	return [lambda, beeta]
end
//___________________________________________________________________________________________________
// Convert from right ascension and declination to galactic coordinates.
//    This implements Meeus equations 13.7 and 13.8.
//
//    Also see:
//    Blaauw, A., Gum, C. S., Pawsey, J. L., & Westerhout, G. (1960).
//    The New I.A.U. System of Galactic Coordinates (1958 Revision).
//    Monthly Notices of the Royal Astronomical Society, 121(2), 123–131.
//    doi:10.1093/mnras/121.2.123
//
//    Available at:
//    http://sci-hub.tw/10.1093/mnras/121.2.123
//
//    arguments:
//      (ra, decl)
//
//    where:
//       ra:   right ascension (can be angle or time)
//       decl: declination
//     both are referred to the the standard equinox of B1950.0.  If a different
//     mean place (e.g. 2000.0) of the star is given instead of the 1950.0 mean
//     place, you need to convert ra and decl to ra_1950 and decl_1950.  Use
//     the "apparentPosition" Function below to perform this conversion.
//     See Meeus 21.2
//
//    returns:
//      (l, b)
//  
//    where:
//       l:  galactic longitude
//       b:  galactic latitude
//     both are referred to the standard equinox of B1950.0.
//
Function [Variable l, variable b] raDeclToGalactic(Variable ra, Variable  decl) 
	
	// Galactic north pole right ascension in 1950 coordinates
	Variable poleRA = 192.25 // degrees
	
	// Galactic north pole declination in 1950 coordinates
	Variable poleDecl = 27.4 // degrees
	
	Variable x = atan2(sin(poleRA - ra), cos(poleRA - ra)*sin(poleDecl) - tan(decl)*cos(poleDecl)) // 13.7
	l = mod(303 - x,360)
	b = asin(sin(decl)*sin(poleDecl) + cos(decl)*cos(poleDecl)*cos(poleRA - ra))  // eq. 13.8
	
	return [l, b] // in degrees
end
//___________________________________________________________________________________________________
// Convert from right ascension and declination to galactic coordinates.
//    This implements the equations after Meeus equations 13.7 and 13.8, p94
//
//    arguments:
//      (l, b)
//
//    where:
//       l:  galactic longitude
//       b:  galactic latitude
//     both of these should be referenced to the standard equinox of B1950.0.
//
//    returns:
//      (ra, decl)
//
//    where:
//       ra:   right ascension
//       decl: declination
//     both are referred to the standard equinox of B1950.0.    If a different
//     mean place (e.g. 2000.0) of the star is desired instead of the 1950.0 mean
//     place, you need to convert ra and decl using 
//     the "apparentPosition" Function below to perform this conversion.
//     See Meeus 21.2
//
Function [Variable alpha, Variable delta] galacticToRADecl(Variable l,Variable  b) 
	
   // Galactic north pole right ascension in 1950 coordinates
   Variable poleRA = 192.25 // degrees

   // Galactic north pole declination in 1950 coordinates
   Variable poleDecl = 27.4 // degrees

   Variable y = atan2(sin(l - 123), cos(l - 123)*sin(poleDecl) - tan(b)*cos(poleDecl))
   alpha = mod(y + 12.25,360)
   delta = asin(sin(b)*sin(poleDecl) + cos(b)*cos(poleDecl)*cos(l - 123))

   return [alpha, delta] // in degrees
end
//___________________________________________________________________________________________________
// Calculate the azimuth and altitude of an object.
// Arguments:
//  (JD, objectRA, objectDecl, lat, long)
//
// date: time of observation
// objectRA:    Right Ascension of object (can be angle or time)
// objectDecl:  Declination of object
// lat:         Latitude of observer (North is positive)
// long:        Longitude of observer (West is positive)
//
// Returns:
//    (azimuth, altitude) as angles.  Azimuth is calculated as angle west from
//                                    south.  To get compass bearing, use
//                                    (azimuth + 180 degrees) mod circle
// 
// Implements equations 13.5 and 13.6 from Meeus.
// NOTE: This is not corrected for refraction nor parallax!  These are
// geocentric coordinates, that is, relative to the center of the earth.
// Due to parallax, the altitude figure may be significantly in error for
// nearby bodies like the moon.  To get the corrected altitudes, use the
// refractedAzimuthAltitude Function below.
Function [Variable azimuth,Variable  altitude] airlessAzimuthAltitude(Variable JD,Variable  objectRA,Variable  objectDecl,Variable  lat,Variable  long) 

	Variable H = rad(hourAngle(JD, long, objectRA))

	azimuth = mod(rad(180) + atan2(sin(H), (cos(H)*sin(rad(lat))) - (tan(objectDecl)*cos(rad(lat)))),rad(360)) // 13.5 
	altitude = asin(sin(rad(lat))*sin(objectDecl) + cos(rad(lat))*cos(objectDecl)*cos(H)) // 13.6
	
	return [azimuth, altitude] // in radians
end
//___________________________________________________________________________________________________
// Calculate the apparent azimuth and altitude of an object with atmospheric
// refraction and parallax corrections.
// Arguments:
//  (JD, objectRA, objectDecl, lat, long, distance, temp, pressure)
//
// date: time of observation
// objectRA:    Right Ascension of object (can be angle or time)
// objectDecl:  Declination of object (angle)
// lat:         Latitude of observer (North is positive)
// long:        Longitude of observer (West is positive)
// distance:    Distance to object (undef means don't correct for parallax)
// temp:        Temperature of atmosphere
// pressure:    Atmospheric pressure
//
// Returns:
//    (azimuth, altitude) as angles.  Azimuth is calculated as angle west from
//                                    south.  To get compass bearing, use
//                                    (azimuth + 180 degrees) mod circle
// 
// Implements equations 13.5 and 13.6 from Meeus.
//
Function [Variable azimuth,Variable corrAltitude] refractedAzimuthAltitude(Variable JD, Variable objectRA, Variable objectDecl,Variable lat, Variable long ,Variable distance,	Variable temp, Variable pressure)	
	
	[azimuth, Variable altitude] = airlessAzimuthAltitude(JD, objectRA, objectDecl, lat, long)
	
	// Correct for parallax
	altitude = altitude - parallaxAngleAlt(distance, altitude)
	corrAltitude = altitude + refractionAngle(altitude, temp, pressure)
	
	return[azimuth, corrAltitude] // in radians
end
//___________________________________________________________________________________________________
// Calculate angle of refraction given the altitude in airless coordinates.
// This is equation 16.4 in Meeus, although the equation in the book
// is explained ridiculously poorly, (which is very uncharacteristic for Meeus'
// extraordinarily good book,) and I had to dig through old 
// Sky & Telescope program listings to figure out how to apply the equation.
// This is the angle to be *added* to the airless altitude to get the
// refracted altitude.
Function refractionAngle(Variable altitude,Variable temp,Variable pressure) 
	
   // h5 is altitude in degrees
   Variable h5 = deg(altitude)  
   Variable v5 = rad(h5 + 10.3/(h5 + 5.11))
   Variable pressCorr = (pressure/1010)*(283/temp)
   Variable R = pressCorr * 0.000296706 / tan(v5)

   return R // in radians
end
//___________________________________________________________________________________________________
// Returns the apparent azimuth and altitude for the sun, adjusted for
// nutation, but not adjusted for refraction.
// 
// Returns:
//   (azimuth, altitude)  Azimuth is calculated as angle west from
//                        south.  To get compass bearing, use
//                        (azimuth + 180 degrees) mod circle
Function [Variable azimuth,Variable altitude] airlessSunAzimuthAltitude(Variable JD, Variable lat, Variable long) 

	[Variable sunRA,Variable sunDecl] = sunApparentRADecl(JD)
	[azimuth, altitude] = airlessAzimuthAltitude(JD, sunRA, sunDecl, lat, long)
	
	return [azimuth, altitude] // in radians
end
//___________________________________________________________________________________________________
// Returns the apparent azimuth and altitude for the sun, adjusted for
// nutation, atmospheric refraction, and parallax.
// 
// Returns:
//   (azimuth, altitude) as angles.  Azimuth is calculated as angle west from
//                                   south.  To get compass bearing, use
//                                   (azimuth + 180 degrees) mod circle
Function [Variable azimuth,Variable altitude] refractedSunAzimuthAltitude(Variable JD, Variable lat, Variable long, Variable temp, Variable pressure) 

	[Variable sunRA,Variable sunDecl] = sunApparentRADecl(JD)
	[azimuth, altitude] = refractedAzimuthAltitude(JD, sunRA, sunDecl, lat, long, sunDistance(JD), temp, pressure)
	
	return [azimuth, altitude] // in radians
end
//___________________________________________________________________________________________________
// Secant method solver to find when the sun is at a given altitude to
// an observer at the specified latitude and longitude on the Earth.
// It takes a first guess at date1 and adds an hour to that to find
// its two starting points.  Thus, to find sunrise, give it a guess
// of sometime around 6 AM local time and a desired altitude of 0 degrees,
// or about -16 arcmin to account for the time that the sun's upper limb
// is just crosses the horizon.  To find sunset, give it a guess
// around 6 PM local time.
// 
// This currently doesn't handle the case where the sun doesn't cross through
// the specified azimuth on that date and may give unpredictable results in
// this case or in the case when the initial guess is far off.
Function sunSecantAltitude(date1, lat, long, desiredAltitude, temperature, pressure) 
	Variable date1, lat, long, desiredAltitude, temperature, pressure
	
	Variable date2 = date1 + (1/24)
	[Variable azimuth1,Variable altitude1] = refractedSunAzimuthAltitude(date1, lat, long, temperature, pressure)
	
	[Variable azimuth2,Variable altitude2] = refractedSunAzimuthAltitude(date2, lat, long, temperature, pressure)
	
	altitude1 = altitude1 - desiredAltitude
	altitude2 = altitude2 - desiredAltitude
	
	int iter
	do
		Variable correction = (altitude1 * (date1 - date2)) / (altitude1-altitude2)
		Variable dat = date1 - correction
		
		if (abs(correction) < (1/86400))
			return dat 
		endif
		
		if (iter == iterations)
			return NaN
		endif
		
		date2 = date1
		date1 = dat
		altitude2 = altitude1
		[azimuth1, altitude1] = refractedSunAzimuthAltitude(dat, lat, long, temperature, pressure)
		altitude1 = altitude1 - desiredAltitude
		iter++
	while (1)
end
//___________________________________________________________________________________________________
// Secant method solver to find when the sun is at a given altitude to
// an observer at the specified latitude and longitude on the Earth.
// This version does *not* correct for refraction.  To correct for refraction,
// use the Function sunSecantAltitude() instead.
// It takes a first guess at date1 and adds an hour to that to find
// its two starting points.  Thus, to find sunrise, give it a guess
// of sometime around 6 AM local time and a desired altitude of 0 degrees,
// or about -16 arcmin to account for the time that the sun's upper limb
// is just crosses the horizon.  To find sunset, give it a guess
// around 6 PM local time.
// 
// This currently doesn't handle the case where the sun doesn't cross through
// the specified azimuth on that date and may give unpredictable results in
// this case or in the case when the initial guess is far off.
Function sunSecantAirlessAltitude(date1, lat, long, desiredAltitude ) 
	Variable date1, lat, long, desiredAltitude
	
	Variable date2 = date1 + (1/24)
	
	[Variable azimuth1,Variable altitude1] = airlessSunAzimuthAltitude(date1, lat, long)
	
	[Variable azimuth2,Variable altitude2] = airlessSunAzimuthAltitude(date2, lat, long)
	
	altitude1 = altitude1 - desiredAltitude
	altitude2 = altitude2 - desiredAltitude
	
	int iter
	do
		Variable correction = (altitude1 * (date1 - date2)) / (altitude1-altitude2)
		Variable dat = date1 - correction
		
		if (abs(correction) < (1/86400))
			return dat 
		endif
		
		if (iter == iterations)
			return NaN
		endif
		
		date2 = date1
		date1 = dat
		altitude2 = altitude1
		[azimuth1, altitude1] = airlessSunAzimuthAltitude(dat, lat, long)
		altitude1 = altitude1 - desiredAltitude
		iter++
	while (1)
end
//___________________________________________________________________________________________________
// Secant method solver to find when the sun is at a given azimuth to
// an observer at the specified latitude and longitude on the Earth.
// It takes a first guess at datee1 and adds an hour to that to find
// its two starting points.  Thus, to find sunrise, give it a guess
// of sometime around 6 AM local time and a desired altitude of 0 degrees,
// or about -16 arcmin to account for the time that the sun's upper limb
// is just crosses the horizon.  To find sunset, give it a guess
// around 6 PM local time.
//
// This Function corrects for refraction.
// 
// This currently doesn't handle the case where the sun doesn't pass through
// the desired azimuth on that date and may give unpredictable results in
// this case or the case where the initial guess is far off.
//
// desiredAzimuth should be specified in the coordinate system specified by
// Meeus, which is angle west of south.  To convert from normal true compass
// bearings, use   (trueAzimuth + 180 degrees) mod circle
Function sunSecantAzimuth(date1, lat, long, desiredAzimuth,temperature,pressure) 
	Variable date1, lat, long, desiredAzimuth, temperature, pressure
		
	Variable date2 = date1 + (5/1440)
		
	[Variable azimuth1,Variable altitude1] = refractedSunAzimuthAltitude(date1, lat, long, temperature, pressure)
		
	[Variable azimuth2,Variable altitude2] = refractedSunAzimuthAltitude(date2, lat, long, temperature, pressure)
		
	azimuth1 = azimuth1 - desiredAzimuth
	azimuth2 = azimuth2 - desiredAzimuth
	
	int iter
	do
		// Don't overcorrect... if we're correcting more than half a phase
		// from the initial guess, then go back a phase.
		if (azimuth1 > rad(180)) 
			azimuth1 = azimuth1 - rad(360)
		endif
		if (azimuth1 < - rad(180)) 
			azimuth1 = azimuth1 + rad(360)
		endif
		if (azimuth2 > rad(180))
			azimuth2 = azimuth2 - rad(360)
		endif
		if (azimuth2 < - rad(180))
			azimuth2 = azimuth2 + rad(360)
		endif
		
		// did we wrap around the circle?
		if (abs(azimuth1 - azimuth2) > rad(180)) 
			if (azimuth1 < azimuth2)
				azimuth2 = azimuth2 - rad(360)
			else
				azimuth1 = azimuth1 - rad(360)
			endif
		endif
		
		Variable correction = (azimuth1 * (date1 - date2)) / (azimuth1-azimuth2)
		Variable dat = date1 - correction
		
		if (abs(correction) < (1/86400))
			return dat 
		endif
		
		if (iter == iterations)
			return NaN
		endif
		
		date2 = date1
		date1 = dat
		azimuth2 = azimuth1
		[azimuth1, altitude1] = refractedSunAzimuthAltitude(dat, lat, long, temperature, pressure)
		azimuth1 = azimuth1 - desiredAzimuth
		iter++
	while (1)
end
//___________________________________________________________________________________________________
// Secant method solver to find when the moon is at a given altitude to
// an observer at the specified latitude and longitude on the Earth.
// It takes a first guess at date1 and adds an hour to that to find
// its two starting points.
// 
// This currently doesn't handle the case where the moon doesn't cross through
// the specified altitude on that date, or if initial guess is very wrong,
// and may give unpredictable results.
Function moonSecantAltitude(date1, lat, long, desiredAltitude,temperature,pressure) 
	Variable date1, lat, long, desiredAltitude,temperature,pressure
	
	Variable date2 = date1 + (1/24)
	
	[Variable azimuth1,Variable altitude1] = refractedMoonAzimuthAltitude(date1, lat, long, temperature, pressure)
	
	[Variable azimuth2,Variable altitude2] = refractedMoonAzimuthAltitude(date2, lat, long, temperature, pressure)
	
	altitude1 = altitude1 - desiredAltitude
	altitude2 = altitude2 - desiredAltitude
	
	int iter
	do
		Variable correction = (altitude1 * (date1 - date2)) / (altitude1-altitude2)

		if (correction > 1)
			correction = 1
		endif
		if (correction < -1)
			correction = -1
		endif
		
		Variable dat = date1 - correction
		if (abs(correction) < (1/86400))
			return dat 
		endif
		
		if (iter == iterations)
			return NaN
		endif
		
		date2 = date1
		date1 = dat
		altitude2 = altitude1
		[azimuth1, altitude1] = refractedMoonAzimuthAltitude(dat, lat, long, temperature, pressure)
		altitude1 = altitude1 - desiredAltitude
		iter++
	while (1)
end
//___________________________________________________________________________________________________
// Calculate the approximate mean time of various phases of the moon.
// This will give the approximate time that the specified phase
// will occur near the given date.   This is only accurate to within
// some number of hours.
//
// dat in year + fractions of a year
//
// The phaseAdjust can be only one of 4 values:
//  0  for New Moon
// 1/4 for First Quarter
// 1/2 for Full Moon
// 3/4 for Last Quarter
//
// All other values give meaningless results!
// 
// This guess serves to supply a good starting guess to moonPhase
// which is much higher accuracy.
 Function moonApproximatePhase(Variable dat,Variable  phaseAdjust) 
	
	// This was found experimentally by Alan Eliasen to give a better initial
	// estimate that centers around the closest phase of the moon, with a
	// slight bias of about 4-5 days toward selecting the "next" event.
	Variable d1 = dat - (phaseAdjust * lunarmonth)
	
	// Get approximate k, Meeus eq. 49.2
	Variable k = round((JDE(d1) - JDE(2451545))/365.25 * 12.3685) + phaseAdjust
	
	Variable T = k / 1236.85 // 49.3
	
	Variable approxdate = JDE(2451550.09766 + 29.530588861*k + 0.00015437*T^2 -  0.000000150*T^3 + 0.00000000073*T^4) // 49.1
	
	return approxdate
end
//___________________________________________________________________________________________________
// This finds the excess of the apparent geocentric latitude of the moon
// over the apparent geocentric longitude of the sun.  It is used to find
// the phases of the moon to medium accuracy.  The accuracy of this is limited
// to the somewhat low accuracy calculation of sunApparentLongitude.
// This could be improved by using the high-accuracy VSOP87 theory
// implemented in planets.frink and Meeus p. 166
// See Meeus chapters 25 and 49.
Function moonExcessLongitude(Variable JD)
	
	Return mod(360 + (moonApparentLongitude(JD) - sunApparentLongitude(JD)),360) // in degrees
end
//___________________________________________________________________________________________________
// Find high accuracy phases of the moon.
// The phaseAdjust can be only one of 4 values:
//  0  for New Moon
// 1/4 for First Quarter
// 1/2 for Full Moon
// 3/4 for Last Quarter
//
// All other values give meaningless results!
// 
// For simplicity, you should probably use one of the convenience Functions:
// newMoon(JD), fullMoon(JD), etc.
//
// This Function uses a secant method successive approximation to find the
// moment of the desired phase to high accuracy using the full-precision
// moon calculations.  The convergence limit is set at one second or less.
Function moonPhase(Variable dat,Variable phaseAdjust) 
	
	Variable desiredAngle = phaseAdjust * 360
	Variable date1 = moonApproximatePhase(dat, phaseAdjust)
	Variable date2 = date1 + (5/1440)
	Variable excess1 = moonExcessLongitude(date1)
	Variable excess2 = moonExcessLongitude(date2)
	
	// Don't overcorrect... if we're correcting more than half a phase forward
	// from the initial guess, then go back a phase.
	// This generally corrects the problem around the new moon where the angle
	// jumps from close to 360 degrees back to 0 degrees, and may prevent other
	// as-yet-undetected weird jumps.
	if ((excess1 - desiredAngle) > 180)
		excess1 -= 360
	endif
	
	if ((excess2 - desiredAngle) > 180)
		excess2 -= 360
	endif
	
	// did we wrap around the circle?
	if (abs(excess1 - excess2) > 180)
		if (excess1 < excess2)
			excess2 -= 360
		else
			excess1 -= 360
		endif
	endif
	
	excess1 -= desiredAngle
	excess2 -= desiredAngle

	do
		Variable correction = (excess1 * (date1 - date2)) / (excess1 - excess2)
	
		dat = date1 - correction
				
		if (abs(correction) < (1/86400)) // second
			return dat
		endif
		
		date2 = date1
		date1 = dat
		excess2 = excess1
		
		excess1 = moonExcessLongitude(date1)
		excess1 -= desiredAngle
		
		// did we wrap around the circle?
		if (abs(excess1 - excess2) > 180)
			if (excess1 < excess2)
				excess2 = excess2 - 360
			else
				excess1 = excess1 - 360
			endif
		endif
	while (1)
end
//___________________________________________________________________________________________________
// Finds the date of the New Moon close to the specifed date.
Function newMoon(Variable JD)
	
	return moonPhase(JD, 0.0)
end
//___________________________________________________________________________________________________
// Finds the date of the First Quarter moon close to the specifed date.
Function firstQuarterMoon(Variable JD)
	
	return  moonPhase(JD, 0.25)
end
//___________________________________________________________________________________________________
// Finds the date of the Full Moon close to the specifed date.
Function fullMoon(Variable JD)
	
	return moonPhase(JD, 0.5)
end
//___________________________________________________________________________________________________
// Finds the date of the Last Quarter moon close to the specifed date.
Function lastQuarterMoon(Variable JD)
	
	return  moonPhase(JD, 0.75)
end
//___________________________________________________________________________________________________
// Secant method solver to find when the moon is at a given azimuth to
// an observer at the specified latitude and longitude on the Earth.
// 
// This currently doesn't handle the case where the moon doesn't pass through
// a certain azimuth on that date and may give unpredictable results in that
// case or the case where the initial guess is far off.
//
// desiredAzimuth should be specified in the coordinate system specified by
// Meeus, which is angle west of south.  To convert from normal true compass
// bearings, use   (trueAzimuth + 180 degrees) mod circle
Function moonSecantAzimuth(date1, lat, long, desiredAzimuth, temperature, pressure)
	Variable date1, lat, long, desiredAzimuth, temperature, pressure
	
	Variable date2 = date1 + (5/1440) // minutes
	[Variable azimuth1,Variable altitude1] = refractedMoonAzimuthAltitude(date1, lat, long, temperature, pressure)
	
	[Variable azimuth2,Variable altitude2] = refractedMoonAzimuthAltitude(date2, lat, long, temperature, pressure)
	
	azimuth1 = azimuth1 - desiredAzimuth
	azimuth2 = azimuth2 - desiredAzimuth
	
	do
		// Don't overcorrect... if we're correcting more than half a phase
		// from the initial guess, then go back a phase.
		if (azimuth1 > rad(180)) // degrees
			azimuth1 = azimuth1 - rad(360)
		endif
		if (azimuth1 < -rad(180)) // degrees
			azimuth1 = azimuth1 + rad(360)
		endif
		if (azimuth2 > rad(180)) // degrees
			azimuth2 = azimuth2 - rad(360)
		endif
		if (azimuth2 < -rad(180)) // degrees
			azimuth2 = azimuth2 + rad(360)
		endif
				
		// did we wrap around the circle?
		if (abs(azimuth1 - azimuth2) > rad(180)) //degrees
			if (azimuth1 < azimuth2)
				azimuth2 = azimuth2 - rad(360)
			else
				azimuth1 = azimuth1 - rad(360)
			endif
		endif
		
		variable correction = (azimuth1 * (date1 - date2)) / (azimuth1-azimuth2)
		Variable dat = date1 - correction
		
		if (abs(correction) < (1/86400)) //second
			return dat
		endif 
		
		date2 = date1
		date1 = dat
		azimuth2 = azimuth1
		[azimuth1, altitude1] = refractedMoonAzimuthAltitude(dat, lat, long, temperature, pressure)
		azimuth1 = azimuth1 - desiredAzimuth
	while (1)
end
//___________________________________________________________________________________________________
// Calculate the time when the upper limb of the sun just crosses the
// horizon.
// Returns undef if the sun does not rise or set on that datee at that latitude.
Function sunrise(dat, lat, long, temp, pressure, altitude)
	Variable dat, lat, long, temp, pressure, altitude
	
	Variable edate = approxMidnight(dat,long) + (5.5/24) // hours
	Variable h_0 = rad(-0.26667) // degrees standard alitiude for the sun
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	[Variable ra,Variable decl] = sunApparentRADecl(edate)
	
	[Variable rise,Variable set] = approxRiseSet(edate, lat, long, ra, decl, h_0)
	
	if (numtype(rise)>0)
		return nan
	else
		return sunSecantAltitude(rise, lat, long, rad(obs_0) + h_0, temp, pressure)
	endif
end
//___________________________________________________________________________________________________
// Calculate the time when the upper limb of the sun just crosses the
// horizon.
// Returns undef if the sun does not rise or set on that date at that latitude.
Function sunset(dat, lat, long, temp, pressure, altitude)
	Variable dat, lat, long, temp, pressure, altitude 
		
	Variable edate = approxMidnight(dat,long) + (17.5/24) // hours
	Variable h_0 = rad(-0.26667) // degrees standard alitiude for the sun
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	[Variable ra,Variable decl] = sunApparentRADecl(edate)
	
	[Variable rise,Variable set] = approxRiseSet(edate, lat, long, ra, decl, h_0)
	
	if (numtype(set)>0)
		return nan
	else
		return sunSecantAltitude(set, lat, long, rad(obs_0) + h_0, temp, pressure)
	endif
end
//___________________________________________________________________________________________________
// Calculate the time of the sun's transit (that is, its highest point in
// the sky.
Function sunTransit(dat, lat, long, temp, pressure)
	Variable dat, lat, long, temp, pressure
				
	Variable approxNoon = approxMidnight(dat, long) + (12/24) // hours
	
	lat=abs(lat)
	
	[Variable az,Variable alt] = refractedSunAzimuthAltitude(approxNoon, lat, long, temp, pressure)
		
	// This is unstable when the sun is directly overhead, so we'll
	// correct a bit (getting the same answer.)
	if (alt > rad(85) ) //degrees
		if (lat > 0) //degrees
			lat += 5 //degrees
		else
			lat -= 5 // degrees
		endif  
		[az, alt] = refractedSunAzimuthAltitude(approxNoon, lat, long, temp, pressure)
	endif

	Variable targetAz
	if (abs(az) < rad(90) ) //degrees    // North of sun, looking south.
		targetAz = 0 //
	else
		targetAz = rad(180) //degrees
	endif

	return sunSecantAzimuth(approxNoon, lat, long, targetAz, temp, pressure)
end
//___________________________________________________________________________________________________
Function GoldenHourBegin(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	return sunSecantAltitude(approxMidnight(JD, long) + (18/24), lat, long, rad(obs_0) + rad(6), temp, pressure )
end
//___________________________________________________________________________________________________
Function civilTwilightBegin(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
 	
 	return sunSecantAltitude(approxMidnight(JD, long) + (5/24), lat, long, rad(obs_0) + rad(-6), temp, pressure )
end
//___________________________________________________________________________________________________
Function civilTwilightEnd(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	return sunSecantAltitude(approxMidnight(JD, long) + (18/24), lat, long, rad(obs_0) + rad(-6), temp, pressure )
end
//___________________________________________________________________________________________________
Function nauticalTwilightBegin(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	return sunSecantAltitude(approxMidnight(JD, long) + (4.7/24), lat, long, rad(obs_0) + rad(-12), temp, pressure )
end
//___________________________________________________________________________________________________
Function nauticalTwilightEnd(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	return sunSecantAltitude(approxMidnight(JD, long) + (18.3/24), lat, long, rad(obs_0) + rad(-12), temp, pressure )
end
//___________________________________________________________________________________________________
Function astronomicalTwilightBegin(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	Return sunSecantAltitude(approxMidnight(JD, long) + (4.3/24), lat, long, rad(obs_0) + rad(-18), temp, pressure )
end
//___________________________________________________________________________________________________
Function astronomicalTwilightEnd(JD, lat, long, temp, pressure, altitude)
	Variable JD, lat, long, temp, pressure, altitude  
	
	Variable obs_0 = ObserverAngle(altitude) // degrees
	
	return sunSecantAltitude(approxMidnight(JD, long) + (18.7/24), lat, long, rad(obs_0) + rad(-18), temp, pressure )
end
//___________________________________________________________________________________________________
// Calculate the darkness of the sky at a specific time.
// The (optional) symbols passed in are an array, containing values that are
// returned for the sun's azimuth when it is:
//
//  0: astronomical twilight ( lower than -18 degrees)
//  1: civil twilight        ( lower than -12 degrees)
//  2: nautical twilight     ( lower than  -6 degrees)
//  3: sunset                ( lower than   0 degrees)
//  4: daylight              ( above 0 degrees) 
//
//  respectively
Function/S skyDarkness(JD, lat, long) 
	Variable JD, lat, long
	String syms = "****;***;**;*;;"
	
	[Variable az,Variable alt] = airlessSunAzimuthAltitude(JD, lat, long)
	if (alt < -18) //degrees
		return stringfromList(0,syms)
	elseif (alt < -12) //degrees
		return stringfromList(1,syms)
	elseif (alt < -6) //degrees
		return stringfromList(2,syms)
	elseif (alt < 0) //degrees
		return stringfromList(3,syms)
	else
		return stringfromList(4,syms)
	endif
end
//___________________________________________________________________________________________________
// Get a time object representing the approximate midnight that begins
// the specified date at the specified longitude.  It is highly recommended
// that you use a date significantly after midnight to eliminate issues with
// the local meridian and daylight savings time. 
Function approxMidnight(Variable JD,Variable longitude)  
	
//	variable d0 = (DateToJulian(2003,01,01) - 0.5) + (longitude / (360/day))
//	variable r = d0 + floor((JD - d0) / day)*day
	
	return floor(JD) - 0.5 - (longitude/360) // edit 20201106
end
//___________________________________________________________________________________________________      
// Besselian date, as defined by B1900
Function Besseliandate(Variable JD) 

	return 1900 + (JD - 2415020.31352) / 365.242198781
end
//___________________________________________________________________________________________________
//
// Moon position calculations
//

// Moon mean longitude L', referred to the mean equinox of date, including the 
// constant term of the effect of the light-time.  This is equation 47.1
// in Meeus.
Function moonMeanLongitude(Variable JD) 
	
   Variable T = meeusT(JD)
      
   return (mod(218.3164477 + 481267.88123421*T - 0.0015786*T^2 + T^3/538841 - T^4/65194000,360))
end
//___________________________________________________________________________________________________
// Moon mean elongation as an angle D.  This is equation 47.2 in Meeus.
Function moonMeanElongation(Variable JD) 
	
   Variable T = meeusT(JD)
   return  (mod(297.8501921 + 445267.1114034*T - 0.0018819*T^2 + T^3/545868 -  T^4/113065000,360))
end
//___________________________________________________________________________________________________
// Calculate the mean anomaly of the sun, M.  This is equation 47.3 in Meeus.
// Note that these numbers are very slightly different than the sun anomaly
// Function given by equation 25.3 and called sunMeanAnomaly above.
Function moonCalcSunMeanAnomaly(Variable JD) 

   Variable T = meeusT(JD)
   return (mod(357.5291092 + 35999.0502909*T - 0.0001536*T^2 + T^3/24490000,360))
end
//___________________________________________________________________________________________________
// Moon mean anomaly M'.  This is equation 47.4 in Meeus.
Function moonMeanAnomaly(Variable JD) 

   Variable T = meeusT(JD)
   return (mod(134.9633964 + 477198.8675055*T + 0.0087414*T^2 + T^3/69699 - T^4 / 14712000,360))
end
//___________________________________________________________________________________________________
// Moon's argument of latitude F, the mean distance of the Moon from its
// ascending node.  This is equation 47.5 in Meeus.
Function moonArgumentOfLatitude(Variable JD) 
	
   Variable T = meeusT(JD)
   return (mod(93.2720950 + 483202.0175233*T - 0.0036539*T^2 - T^3 / 3526000 + T^4 / 863310000, 360))
end
//___________________________________________________________________________________________________
// Moon correction term A1 (listed after equation 47.5 in Meeus)
// This is due to the action of Venus.
Function moonA1(Variable JD)  
	
   Variable T = meeusT(JD)
   return (mod(119.75 + 131.849*T,360))
end
//___________________________________________________________________________________________________
// Moon correction term A2 (listed after equation 47.5 in Meeus)
// This is due to the action of Jupiter.
Function moonA2(Variable JD)  
	
   Variable T = meeusT(JD)
   return (mod(53.09 + 479264.290*T,360))
end
//___________________________________________________________________________________________________
// Moon correction term A3 (listed after equation 47.5 in Meeus)
Function moonA3(Variable JD)  
	
   Variable T = meeusT(JD)
   return (mod(313.45 + 481266.484*T, 360))
end
//___________________________________________________________________________________________________
// Earth eccentricity E for Moon calculations.  This is equation 47.6 in 
// Meeus.  This is not the eccentricity given by eq. 25.4
Function moonCalcEarthEccentricity(Variable JD) 
	
   Variable T = meeusT(JD)
   return 1 - 0.002516*T - 0.0000074*T^2
end
//___________________________________________________________________________________________________
// Moon sum of L from Meeus, chapter 47.  This is with the extra additive
// terms listed on p. 342.
Function moonSumL(Variable JD)  
	
	Variable D = rad(moonMeanElongation(JD))
	Variable M = rad(moonCalcSunMeanAnomaly(JD))
	Variable MP = rad(moonMeanAnomaly(JD))
	Variable F = rad(moonArgumentOfLatitude(JD))
	Variable E = moonCalcEarthEccentricity(JD)
	Variable LP = rad(moonMeanLongitude(JD))
	Variable A1 = rad(moonA1(JD))
	Variable A2 = rad(moonA2(JD))
	
	Variable sumL = ((6288774)*sin(MP) + (1274027)*sin(2*D - MP) + (658314)*sin(2*D) + \
	(213618)*sin(2*MP) + (-185116*E)*sin(M) + (-114332)*sin(2*F) + (58793)*sin(2*D -2*MP) + \
	(57066*E)*sin(2*D - M - MP) + (53322)*sin(2*D + MP) + (45758*E)*sin(2*D - M) + \
	(-40923*E)*sin(M - MP) + (-34720)*sin(D) + (-30383*E)*sin(M + MP) + \
	(15327)*sin(2*D -2*F) + (-12528)*sin(MP + 2*F) + (10980)*sin(MP -2*F) + \
	(10675)*sin(4 *D - MP) + (10034)*sin(3*MP) + (8548)*sin(4*D -2*MP) + \
	(-7888*E)*sin(2*D + M - MP) + (-6766*E)*sin(2*D + M) + (-5163)*sin(D - MP) + \
	(4987*E)*sin(D + M) + (4036*E)*sin(2*D - M + MP) + (3994)*sin(2*D + 2*MP) + \
	(3861)*sin(4*D) + (3665)*sin(2*D -3*MP) + (-2689*E)*sin(M -2*MP) + \
	(-2602)*sin(2*D - MP + 2*F) + (2390*E)*sin(2*D - M -2*MP) + (-2348)*sin(D + MP) + \
	(2236*E^2)*sin(2*D -2*M) + (-2120*E)*sin(M + 2*MP) + (-2069*E^2)*sin(2*M) + \
	(2048*E^2)*sin(2*D -2*M - MP) + (-1773)*sin(2*D + MP -2*F) + (-1595)*sin(2*D + 2*F) + \
	(1215*E)*sin(4*D - M - MP) + (-1110)*sin(2*MP + 2*F) + (-892)*sin(3*D - MP) + \
	(-810*E)*sin(2*D + M + MP) + (759*E)*sin(4*D - M -2*MP) + (-713*E^2)*sin(2*M - MP) + \
	(-700*E^2)*sin(2*D + 2*M - MP) + (691*E)*sin(2*D + M -2*MP) + (596*E)*sin(2*D - M -2*F) + \
	(549)*sin(4*D + MP) + (537)*sin(4*MP) + (520*E)*sin(4*D - M) + (-487)*sin(D -2*MP) + \
	(-399*E)*sin(2*D + M -2*F) + (-381)*sin(2*MP -2*F) + (351*E)*sin(D + M + MP) + \
	(-340)*sin(3*D -2*MP) + (330)*sin(4*D -3*MP) + (327*E)*sin(2*D - M + 2*MP) + \
	(-323*E^2)*sin(2*M + MP) + (299*E)*sin(D + M - MP) + (294)*sin(2*D + 3*MP))
	
	sumL += 3958*sin(A1) + 1962*sin(LP - F) + 318*sin(A2)
	
	return sumL
end
//___________________________________________________________________________________________________
// Moon longitude calculation from section 47 of Meeus.  This is not corrected
// for Earth's nutation.  For that, use moonApparentLongitude(JD)
Function moonLongitude(Variable JD) 
	
	return moonMeanLongitude(JD) + (moonSumL(JD)/1000000)
end
//___________________________________________________________________________________________________
// Returns the moon's apparent longitude, corrected for Earth's
// nutation.  It is not corrected for refraction.
Function moonApparentLongitude(Variable JD) 
	
	[Variable deltaPsi,Variable deltaEpsilon] = highAccuracyNutation(JD)
			
	return moonLongitude(JD) + deltaPsi
end
//___________________________________________________________________________________________________
// Moon distance calculation Delta from section 47 of Meeus.
Function moonDistance(Variable JD) 
	
	Variable D = rad(moonMeanElongation(JD))
	Variable M = rad(moonCalcSunMeanAnomaly(JD))
	Variable MP = rad(moonMeanAnomaly(JD))
	Variable F = rad(moonArgumentOfLatitude(JD))
	Variable E = moonCalcEarthEccentricity(JD)
	Variable LP = rad(moonMeanLongitude(JD))
	
	Variable sumR = ((-20905355)*cos(MP) + (-3699111)*cos(2*D - MP) + (-2955968)*cos(2*D) + \
	(-569925)*cos(2*MP) + (48888*E)*cos(M) + (-3149)*cos(2*F) + \
	(246158)*cos(2*D -2*MP) + (-152138*E)*cos(2*D - M - MP) + (-170733)*cos(2*D + MP) + \
	(-204586*E)*cos(2*D - M) + (-129620*E)*cos(M - MP) + (108743)*cos(D) + \
	(104755*E)*cos(M + MP) + (10321)*cos(2*D -2*F) + (79661)*cos(MP -2*F) + \
	(-34782)*cos(4*D - MP) + (-23210)*cos(3*MP) + (-21636)*cos(4*D -2*MP) + \
	(24208*E)*cos(2*D + M - MP) + (30824*E)*cos(2*D + M) + (-8379)*cos(D - MP) + \
	(-16675*E)*cos(D + M) + (-12831*E)*cos(2*D - M + MP) + (-10445)*cos(2*D + 2*MP) + \
	(-11650)*cos(4*D) + (14403)*cos(2*D -3*MP) + (-7003*E)*cos(M -2*MP) + \
	(10056*E)*cos(2*D - M -2*MP) + (6322)*cos(D + MP) + (-9884*E^2)*cos(2*D -2*M) + \
	(5751*E)*cos(M + 2*MP) + (-4950*E^2)*cos(2*D -2*M - MP) + (4130)*cos(2*D + MP -2*F) + \
	(-3958*E)*cos(4*D - M - MP) + (3258)*cos(3*D - MP) + (2616*E)*cos(2*D + M + MP) + \
	(-1897*E)*cos(4*D - M -2*MP) + (-2117*E^2)*cos(2*M - MP) + (2354*E^2)*cos(2*D + 2*M - MP) + \
	(-1423)*cos(4*D + MP) + (-1117)*cos(4*MP) + (-1571*E)*cos(4*D - M) + (-1739)*cos(D -2*MP) + \
	(-4421)*cos(2*MP -2*F) + (1165*E^2)*cos(2*M + MP) + (8752)*cos(2*D - MP -2*F))
	
	return (385000560 + sumR) //meter
end
//___________________________________________________________________________________________________
// Moon latitude calculation beta from section 47 of Meeus.
Function moonLatitude(Variable JD) 
	
	Variable D = rad(moonMeanElongation(JD))
	Variable M = rad(moonCalcSunMeanAnomaly(JD))
	Variable MP = rad(moonMeanAnomaly(JD))
	Variable F = rad(moonArgumentOfLatitude(JD))
	Variable E = moonCalcEarthEccentricity(JD)
	Variable LP = rad(moonMeanLongitude(JD))
	Variable A1 = rad(moonA1(JD))
	Variable A3 = rad(moonA3(JD))
	
	Variable sumB = ((5128122)*sin(F) + (280602)*sin(MP + F) + \
	(277693)*sin(MP - F) + (173237)*sin(2*D - F) + (55413)*sin(2*D - MP + F) + \
	(46271)*sin(2*D - MP - F) + (32573)*sin(2*D + F) + (17198)*sin(2*MP + F) + \
	(9266)*sin(2*D + MP - F) + (8822)*sin(2*MP - F) + (8216*E)*sin(2*D - M - F) + \
	(4324)*sin(2*D -2*MP - F) + (4200)*sin(2*D + MP + F) + (-3359*E)*sin(2*D + M - F) + \
	(2463*E)*sin(2*D - M - MP + F) + (2211*E)*sin(2*D - M + F) + (2065*E)*sin(2*D - M - MP - F) + \
	(-1870*E)*sin(M - MP - F) + (1828)*sin(4*D - MP - F) + (-1794*E)*sin(M + F) + \
	(-1749)*sin(3*F) + (-1565*E)*sin(M - MP + F) + (-1491)*sin(D + F) + (-1475*E)*sin(M + MP + F) + \
	(-1410*E)*sin(M + MP - F) + (-1344*E)*sin(M - F) + (-1335)*sin(D - F) + \
	(1107)*sin(3*MP + F) + (1021)*sin(4*D - F) + (833)*sin(4*D - MP + F) + (777)*sin(MP -3*F) + \
	(671)*sin(4*D -2*MP + F) + (607)*sin(2*D -3*F) + (596)*sin(2*D + 2*MP - F) + \
	(491*E)*sin(2*D - M + MP - F) + (-451)*sin(2*D -2*MP + F) + (439)*sin(3*MP - F) + \
	(422)*sin(2*D + 2*MP + F) + (421)*sin(2*D -3*MP - F) + (-366*E)*sin(2*D + M - MP + F) + \
	(-351*E)*sin(2*D + M + F) + (331)*sin(4*D + F) + (315*E)*sin(2*D - M + MP + F) + \
	(302*E^2)*sin(2*D -2*M - F) + (-283)*sin(MP + 3*F) + (-229*E)*sin(2*D + M + MP - F) + \
	(223*E)*sin(D + M - F) + (223*E)*sin(D + M + F) + (-220*E)*sin(M -2*MP - F) + \
	(-220*E)*sin(2*D + M - MP - F) + (-185)*sin(D + MP + F) + (181*E)*sin(2*D - M -2*MP - F) + \
	(-177*E)*sin(M + 2*MP + F) + (176)*sin(4*D -2*MP - F) + (166*E)*sin(4*D - M - MP - F) + \
	(-164)*sin(D + MP - F) + (132)*sin(4*D + MP - F) + (-119)*sin(D - MP - F) + (115*E)*sin(4*D - M - F) + \
	(107*E^2)*sin(2*D -2*M + F))

	sumB = sumB - 2235*sin(LP) + 382*sin(A3) + 175*sin(A1 - F) + 175*sin(A1 + F) + 127*sin(LP - MP) - 115*sin(LP + MP)

	return sumB / 1000000 // degree
end
//___________________________________________________________________________________________________
// Calculates (numerically) the moon's radial velocity towards earth, in m/s.
Function moonRadialVelocity(Variable JD) 
	
	Variable halfStep = (0.5 / 86400) // s
	Variable d1 = JD - halfStep
	Variable d2 = JD + halfStep
	
	return (moonDistance(d1) - moonDistance(d2)) / ((halfStep * 2)*86400)
end
//___________________________________________________________________________________________________
// Calculates the apparent Right ascension and declination of the moon
// at a given date.
// returns (ra, decl)
Function [Variable ra, Variable  decl] moonApparentRADecl(Variable JD) 
	
	Variable epsilon = rad(trueObliquityOfEcliptic(JD))
	
	Variable moonLong = rad(moonApparentLongitude(JD))
	Variable moonLat = rad(moonLatitude(JD))
	Variable se = sin(epsilon)
	Variable ce = cos(epsilon)
	Variable sm = sin(moonLong)
	ra = mod(rad(360) + atan2(sm*ce - tan(moonLat)*se, cos(moonLong)),rad(360))
	decl = mod(rad(360) + asin(sin(moonLat)*ce + cos(moonLat)*sin(epsilon)*sm),rad(360))
	
	return [ra,decl]
end
//___________________________________________________________________________________________________
// Returns the apparent azimuth and altitude for the moon, adjusted for
// nutation, but not adjusted for refraction.
// 
// Returns:
//   (azimuth, altitude)  Azimuth is calculated as angle west from
//                        south.  To get compass bearing, use
//                        (azimuth + 180 degrees) mod circle
Function [Variable azimuth,Variable altitude] airlessMoonAzimuthAltitude(Variable JD,Variable  lat,Variable  long) 
	
	[Variable moonRA,Variable moonDecl] = moonApparentRADecl(JD)
	
	[azimuth, altitude] = airlessAzimuthAltitude(JD, moonRA, moonDecl, lat, long)
	
	return [azimuth, altitude]
end
//___________________________________________________________________________________________________
// Returns the apparent azimuth and altitude for the moon, adjusted for
// nutation, atmospheric refraction, and parallax.
// 
// Returns:
//   (azimuth, altitude) as angles.  Azimuth is calculated as angle west from
//                                   south.  To get compass bearing, use
//                                   (azimuth + 180 degrees) mod circle
Function [Variable azimuth, Variable  altitude] refractedMoonAzimuthAltitude(Variable JD,Variable  lat,Variable  long,Variable  temp,Variable  pressure) 

   [Variable moonRA,Variable moonDecl] = moonApparentRADecl(JD)

	[azimuth,altitude] = refractedAzimuthAltitude(JD, moonRA, moonDecl, lat, long, moonDistance(JD), temp, pressure)
   return [azimuth,altitude]
end
//___________________________________________________________________________________________________
// Calculates the apparent moon radius angle, corrected for distance, but not
// corrected for refraction.  It gives a true arcsin geometric interpretation.
Function moonRadiusAngle(Variable JD)  
	
	return asin(moonradius / (moonradius + moonDistance(JD)))
end
//___________________________________________________________________________________________________
// Calculates the geocentric elongation of the moon from the sun
// by Meeus equation 48.2 (second equation)
Function moonGeocentricElongationFromSun(Variable JD)  
	
   return acos(cos(rad(moonLatitude(JD))) * cos(rad(moonApparentLongitude(JD)) - rad(sunTrueLongitude(JD))))
end
//___________________________________________________________________________________________________
// Calculates the geocentric elongation of the moon from the sun
// by Meeus equation 48.2 (first equation)
Function moonGeocentricElongationFromSun2(Variable JD)  
	
   [Variable alpha0,Variable delta0] = sunApparentRADecl(JD)
   
   [Variable alpha,Variable delta] = moonApparentRADecl(JD)
   
   return acos(sin(delta0)*sin(delta) + cos(delta0)*cos(delta)*cos(alpha0 - alpha))
end
//___________________________________________________________________________________________________
// Phase angle of the moon as seen from earth, by equation 48.3 of Meeus
// This is constrained to be in the range 0-180 degrees as expected by the
// illuminated fraction calculations.
 Function moonPhaseAngle(Variable JD) 

   Variable R = sunDistance(JD)
   Variable delta = moonDistance(JD)
   Variable phi = moonGeocentricElongationFromSun(JD)

   return mod(rad(180)+(atan(R*sin(phi) / (delta - R*cos(phi)))),rad(180)) 
end
//___________________________________________________________________________________________________
// Illuminated fraction of the moon as seen from earth.  This is found by
// equation 48.1 of Meeus.
Function moonIlluminatedFraction(Variable JD)

	Return  (1 + cos(moonPhaseAngle(JD))) / 2
end
//___________________________________________________________________________________________________
// The position angle of the moon's bright limb, reckoned eastward from the
// North Point of the moon.  (That is, relative to the earth's North/South
// axis projected onto the moon's disc, *not* relative to the moon's axis.)
// This is equation 48.5 in Meeus.
Function moonPositionAngle(Variable JD) 
	
   [Variable alpha0,Variable delta0] = sunApparentRADecl(JD)
   
   [Variable alpha,Variable delta] = moonApparentRADecl(JD)
   
   return atan2(cos(delta0)*sin(alpha0 - alpha), sin(delta0)*cos(delta) - cos(delta0)*sin(delta)*cos(alpha0-alpha))
end
//___________________________________________________________________________________________________
// The position angle of the moon's bright limb, reckoned "eastward" (?)
// from the zenith.  Note that this gives the direction *counterclockwise*
// from the zenith as seen from an observer standing on earth.  The "eastward"
// terminology is confusing.
// This is equation 48.5 in Meeus.
Function moonPositionAngleRelativeToZenith(Variable JD,Variable lat,Variable long) 
	
   Variable chi = moonPositionAngle(JD)
      
   [Variable ra,Variable decl] = moonApparentRADecl(JD)
   Variable q = parallacticAngle(JD, lat, long, ra, decl)
   
   return chi - q
end
//___________________________________________________________________________________________________
// The position angle of the moon's axis of rotation (this is *not* the
// illuminated axis,) reckoned eastward from the zenith.
// from the zenith.  Note that this gives the direction *counterclockwise*
// from the zenith as seen from an observer standing on earth.  The "eastward"
// terminology is confusing.
Function moonAxisAngleRelativeToZenith(JD, lat, long, heightAboveSeaLevel)
	Variable JD, lat, long, heightAboveSeaLevel

   Variable axis = moonAxisAngle(JD, lat, long, heightAboveSeaLevel)
   
   [Variable ra,Variable decl] = moonApparentRADecl(JD)
   
   Variable q = parallacticAngle(JD, lat, long, ra, decl)
   return axis - q
end
//___________________________________________________________________________________________________

// The position angle of the moon's axis of rotation (this is *not* the
// illuminated axis,) reckoned eastward from the
// North Point of the moon.  (That is, relative to the earth's North/South
// axis projected onto the moon's disc.)
// See chapter 53 in Meeus. and the moonLibrationAndAxis formula.
Function moonAxisAngle(JD, lat, long, heightAboveSeaLevel) 
	Variable JD, lat, long, heightAboveSeaLevel
   // TODO:  pass in lambda and beta for lat, long
   // See Meeus chapter 53, "Topocentric Librations" section
   [Variable ra,Variable decl] = moonApparentRADecl(JD)
   
   Variable distance = moonDistance(JD)
   
   [Variable ra1,Variable decl1] = correctRADeclForParallax(JD, ra, decl, lat, long, distance, heightAboveSeaLevel)
   Variable epsilon = trueObliquityOfEcliptic(JD)
   
   [Variable lambda,Variable beeta] = raDeclToEcliptical(ra1, decl1, epsilon)
   
   [Variable l,Variable b,Variable P] = moonLibrationAndAxis(JD, lambda, beeta)
   
   return P
end
//___________________________________________________________________________________________________
// Parallax angle corrections (in azimuth and altitude) of an object as seen
// from earth.  This is needed because most of the other calculations here
// are geocentric, i.e., taken relative to the center of the earth.  The
// parallax becomes especially significant when taken to objects like the
// moon which are close to earth, and when looking at them close to the
// horizon.  Sigh.  I found this out the hard way.
// RETURNS:
//  altParallax
Function parallaxAngleAlt(Variable distance,Variable origAltitude) 
	
   // TODO:  Add a term for altitude above the reference geoid
   
   // Horizontal parallax, called pi in Meeus
   // This is the equation *before* equation 40.1 in Meeus.
   // azParallax = asin(8.794 arcsec/(distance/au))

   // The following possibly gives more accuracy and is more general
   // without the random unexplained factors of Meeus.
   Variable azParallax = asin(earthradius_equatorial/distance)

   // TODO:  Correct this for non-sphericity of Earth.  For now, rho
   // is taken to be 1.
   Variable altParallax = asin(sin(azParallax)*cos(origAltitude))

   return altParallax
end
//___________________________________________________________________________________________________

// Parallax angle corrections (in right ascension and declination) of an object
// as seen from earth.  This is needed because most of the other calculations
// here are geocentric, i.e., taken relative to the center of the earth.  The
// parallax becomes especially significant when taken to objects like the
// moon which are close to earth, and when looking at them close to the
// horizon.
// RETURNS:
//  (ra, decl)  corrected for parallax
Function [Variable raPrime, Variable declPrime] correctRADeclForParallax(Variable JD,Variable ra,Variable decl,Variable lat,Variable long,Variable distance,Variable heightAboveSeaLevel) 
	
	// This is more clear than eq. 40.1
	Variable azParallax = asin(earthradius_equatorial / distance)
	
	[Variable rsp,Variable rcp] = rhoSinPhi(lat, heightAboveSeaLevel)
	
	Variable H = hourAngle(JD, long, ra)
	Variable sp = sin(azParallax)
	Variable deltaRA = atan2(-rcp*sp*sin(H), cos(decl) - rcp*sp*cos(H)) // Eq 40.2
	raPrime = ra + deltaRA
	declPrime = atan2((sin(decl) - rsp*sp)*cos(deltaRA), cos(decl) - rcp*sp*cos(H)) // Eq 40.3
	
	return [raPrime, declPrime]
end
//___________________________________________________________________________________________________
// Calculate the values of rho sin phi' and rho cos phi' used in parallax
//    calculations.  These are defined in chapter 11 of Meeus, and used in
//    parallax calculations (chapter 40.)
//
//    Arguments:
//       (lat, heightAboveSeaLevel)
//
//      where
//         lat: the ordinary geographical latitude (phi)
//
//    returns:
//     (rho sin phi', rho cos phi')
//
Function [Variable rhoSinPhi, Variable rhoCosPhi] rhoSinPhi(Variable lat, Variable heightAboveSeaLevel) 

	Variable a = earthradius_equatorial
	Variable f = earth_flattening
	Variable b = a*(1-f)   // Earth polar radius
	Variable bOvera = b/a
	
	Variable u = atan(bOvera*tan(lat))
	rhoSinPhi = bOvera*sin(u) + heightAboveSeaLevel/a*sin(lat)
	rhoCosPhi = cos(u) + heightAboveSeaLevel/a*cos(lat)
	
	return [rhoSinPhi, rhoCosPhi]   
end
//___________________________________________________________________________________________________
// Returns the angular separation between two bodies given their right
// ascensions and declinations (ra1, decl1, ra2, decl2)
//   Note that when using right ascension and declination, these are geocentric
// coordinates and do not take parallax into account, so they are useless for
// eclipses or transit predictions or close alignments with the moon and other
// objects.
//
// It also works for altazimuth coordinates if passed: (az1, alt1, az2, alt2)
// In the altazimuth case, you can correct for parallax and refraction from
// a particular point on the earth, and if you did so, you may use this to
// predict eclipses, transits, etc. successfully.
Function angularSeparation(ra1, decl1, ra2, decl2) 
	Variable ra1, decl1, ra2, decl2
	
	Variable arg = sin(decl1)*sin(decl2) + cos(decl1)*cos(decl2)*cos(ra1-ra2)
	
	// We want to condition the argument to allow intervals, as typical
	// intervals wildly overestimate to the point that the values aren't within
	// the domain of arccos.  We'll constrain arguments to the domain of arccos,
	// (-1,1)

	if (arg < -1)
		Arg=-1
	elseif (arg > 1)
		Arg=1
	endif
	
	return acos(Arg)
end
//___________________________________________________________________________________________________
// Convenience Function to obtain the angular separation between the sun
// and moon for a given time.
// WARNING:  This is the GEOCENTRIC angular separation, that is, as seen from
// the center of the earth and is not corrected for refraction or parallax,
// so it is useless for, say, predicting eclipses or transits!
// In other words, this is probably not the Function you're looking for.
// You most likely want to use the Function below that requires latitude and
// longitude.
Function geocentricSunMoonAngularSeparation(Variable JD) 
	
   [Variable sunRA,Variable sunDecl] = sunApparentRADecl(JD)
   
   [Variable moonRA,Variable moonDecl] = moonApparentRADecl(JD)
  
   return angularSeparation(sunRA, sunDecl, moonRA, moonDecl)
end
//___________________________________________________________________________________________________
// This returns the angular separation of the sun and moon as seen from
// a particular point on the earth.  It is corrected for parallax.
Function sunMoonAngularSeparation(Variable JD,Variable lat,Variable long, Variable temp, Variable pressure) 
	
   [Variable sunaz,Variable sunalt]  =  refractedSunAzimuthAltitude(JD, lat, long, temp, pressure)
   
   [Variable moonaz,Variable moonalt] = refractedMoonAzimuthAltitude(JD, lat, long, temp, pressure)
  
   return angularSeparation(sunaz, sunalt, moonaz, moonalt)
end
//___________________________________________________________________________________________________
// Calculates the approximate transit time of a body which has the specified
// right ascension and declination.  The resultant value will be close
// to the specified date.  See Section 15.2 of Meeus.
// This value should not be used directly, but should be used as a seed in
// something like moonTransit()
Function approxTransit(Variable JD,Variable long,Variable ra,Variable decl) 
	
	Variable m0 = (ra - rad(apparentLocalSiderealAngle(JD, long))) / rad(360) //degrees
	m0 = mod(1+m0,1) 

	if (m0 > 0.5) 
		m0 = m0 - 1
	endif
	if (m0 < -0.5) 
		m0 = m0 + 1 
	endif

	return JD + m0
end
//___________________________________________________________________________________________________
// Calculate the approximate offset of rise/set for a body with the specified
// right ascension and declination as seen from the specified latitude.
// h0 is the "standard altitude" with refraction.  The default value -0.5667 is
// accurate for point objects, or -0.8333 degrees should be used for the sun.
//
//  Returns hour angle or undef if the object is circumpolar.
//  THINK ABOUT:  Throw an exception?
Function calcHourAngle(Variable lat,Variable decl,Variable h_0) 
	
	lat=rad(lat)
	
	Variable cosH0 = (sin(h_0) - sin(lat)*sin(decl)) / (cos(lat)*cos(decl))
   
   if (abs(cosH0) > 1)
      return nan
   else
      return acos(cosH0) // degrees
   endif
end
//___________________________________________________________________________________________________
// Calculates the approximate rise and set time of a body which has the
// specified right ascension and declination.  The resultant value will be
// close to the specified date.  See Equations 15.1 - 15.2 of Meeus.
// This value should not be used directly, but should be used as a seed in
// something like moonTransit()
//
//  returns:
//    (rise, set)
//  Return values are undef if the object is circumpolar for that date.
Function [Variable rise, Variable set] approxRiseSet(Variable JD, Variable lat, Variable long,Variable  ra, Variable decl, Variable h_0) 
	   
   Variable transit = approxTransit(JD, long, ra, decl)
   Variable H0 = calcHourAngle(lat, decl, h_0)
   
   if (numtype(H0)>0)
      return [nan, nan]
      endif
   
   Variable Htime = H0 / rad(360) // degrees
   rise =  transit - Htime
   set = transit + Htime

   return [rise,set]
end
//___________________________________________________________________________________________________
// Calculate the time when the upper limb of the moon just crosses the
// horizon.
// Returns undef if the sun does not rise or set on that date at that latitude.
Function moonrise(JD, lat, long, temp, pressure, altitude) 
	Variable JD, lat, long, temp, pressure, altitude  

	Variable h_0 = rad(-0.266667) // degrees standard altiude for stars and planets at rise time
	Variable obs_0 = ObserverAngle(altitude)
	
   [Variable ra,Variable decl] = moonApparentRADecl(JD)
   
   [Variable rise,Variable set] = approxRiseSet(JD, lat, long, ra, decl, h_0)

   if (numtype(rise)>0)
      return nan
   else
      return moonSecantAltitude(rise, lat, long, rad(obs_0) + h_0, temp, pressure)
   endif
end
//___________________________________________________________________________________________________
// Calculate the time when the upper limb of the moon just crosses the
// horizon.
// Returns undef if the sun does not rise or set on that date at that latitude.
Function moonset(JD, lat, long, temp, pressure, altitude) 
	Variable JD, lat, long, temp, pressure, altitude  

	Variable h_0 = rad(-0.266667) // degrees standard altiude for stars and planets at rise time
	Variable obs_0 = ObserverAngle(altitude)
	
	[Variable ra,Variable decl] = moonApparentRADecl(JD)
	
	[Variable rise,Variable set] = approxRiseSet(JD, lat, long, ra, decl,h_0)
	
	if (numtype(set)>0)
		return nan
	else
		return moonSecantAltitude(set, lat, long, rad(obs_0) + h_0, temp, pressure)
	endif
end
//___________________________________________________________________________________________________
// Calculate the time of the moon's transit (that is, its highest point in
// the sky.
Function moonTransit(Variable JD,Variable lat,Variable long,Variable temp,Variable pressure)
	
	[Variable moonRA,Variable moonDecl] = moonApparentRADecl(JD)
	
	Variable approxTrans = approxTransit(JD, long, moonRA, moonDecl) 
	
	lat=abs(lat)
	
	[Variable az,Variable alt] = refractedMoonAzimuthAltitude(approxTrans, lat, long, temp, pressure)

	// This is unstable when the moon is directly overhead, so we'll
	// correct a bit (getting the same answer.)
	if (alt > rad(85)) // degrees
		if (lat > 0)// degrees
			lat = lat + 5 // degrees
		else
			lat = lat - 5 // degrees
		endif   
		
		[az,alt] = refractedMoonAzimuthAltitude(approxTrans, lat, long, temp, pressure)
	endif
	
	Variable targetAz
	if (abs(az) < rad(90)) // degrees    // North of moon, looking south.
		targetAz = 0 //degrees
	else
		targetAz = rad(180) //degrees
	endif

	return moonSecantAzimuth(approxTrans, lat, long, targetAz, temp, pressure)
end
//___________________________________________________________________________________________________
// Calculate the dip angle of the horizon.  This is a simplified calculation
// that just takes the height above the average sphere.
Function horizonDip(Variable height) 
	
	return acos(earthradius / (earthradius+height))
end
//___________________________________________________________________________________________________
// Calculate the time of perigee of the moon closest to the specified date.
//  Returns:  The date of perigee.
Function moonPerigee(Variable JD) 
	
	return moonMeanApogeeOrPerigeeInternal(JD, 0)
end
//___________________________________________________________________________________________________
// Calculate the time of apogee of the moon closest to the specified date.
//  Returns:  The date of apogee.
Function moonApogee(Variable JD) 
	
	return moonMeanApogeeOrPerigeeInternal(JD, 0.5)
end
//___________________________________________________________________________________________________
// Estimate the apogee or perigee of the moon closest to a particular
// date.  See Meeus chapter 50.  This should not be called directly by the
// user.  Its arguments should be preconditioned by calling
// moonApogee(JD) or moonPerigee(JD)
Function moonMeanApogeeOrPerigeeInternal(Variable dat,Variable correctionFactor) 
	
	Variable epoch = dateToJulian(1999,12,22) - 0.5 // UTC   // Definition of k=0 in Meeus
	
	// Round to closest integer or half-integer.
	Variable k = round((((dat - epoch) / 365.25) * 13.2555) + correctionFactor) - correctionFactor
	
	Variable T = k/1325.55    // Meeus 50.3
	
	Variable dateph = JDE(2451534.6698 + 27.55454989*k - 0.0006691*T^2 - 0.000001098*T^3 + 0.0000000052*T^4)
	
	// We now use the secant method to find when the moon's radial velocity
	// relative to earth becomes zero around this point.  This doesn't use the
	// rest of the procedure found in Meeus ch. 50, but rather allows us to
	// later plug in a more accurate moon position (such as the *FULL* ELP-2000
	// model) and this will automatically get more accurate.
	
	Variable d1 = dateph
	Variable d2 = dateph + (0.1/86400) // s
	Variable v1 = moonRadialVelocity(d1)
	Variable v2 = moonRadialVelocity(d2)
	do
	
		Variable correction = (v1 * (d1 - d2)) / (v1 - v2)
		Variable dnew = d1 - correction
		
		if (abs(correction) < (1/86400)) // second
			return d1 - correctionFactor
		endif
		
		d2 = d1
		d1 = dnew
		v2 = v1
		v1 = moonRadialVelocity(d1)
	while (1)
end
//___________________________________________________________________________________________________
// Calculate the longitude of the mean ascending node (Omega).
//    This is equation 47.7 in Meeus. 
Function moonMeanLongitudeOfAscendingNode(Variable JD) 

	Variable T = meeusT(JD)
	
	return (125.0445479 - 1934.1362891*T + 0.0020754*T^2 + T^3 / 467441 - T^4 / 60616000) // degrees
end
//___________________________________________________________________________________________________
// Calculate the optical libration of the moon.   This is chapter 53 in
//    Meeus.  It also calculates the position angle of the moon's axis because
//    it requires all of these calculations
//
//    The parameters lambda and beta should be left as undef if you want the
//    geocentric libration value (That is, as seen from the center of the
//    earth.)
//
//    They are allowed to be passed in if you want the libration for
//    an exact position on the earth.  See "Topocentric Librations" in chapter
//    53 to see how to do this.  In this case, lambda should be the apparent
//    geocentric longitude and beta the apparent geocentric latitude of the moon.
//
//    returns: (l, b, P)
//      l:  physical libration in longitude
//      b:  physical libration in latitude
//      P:  the position angle of the Moon's axis of rotation  (See the diagram
//          in Chapter 42 of Meeus)
//
Function [Variable l,Variable  b,Variable  P] moonLibrationAndAxis(Variable JD, Variable lambda, Variable beeta) 
	
	// Inclination of the mean lunar equator to the ecliptic
	Variable I = 1.54242 // degrees
	
	[Variable deltaPsi,Variable deltaEpsilon] = highAccuracyNutation(JD)
	
	// We expect the apparent longitude to have been passed in (that is,
	// longitude corrected for nutation) so we subtract out the nutation.
	Variable lambdaMinusNutation = lambda - deltaPsi
	
	// First, calculate the optical librations.  These are the larger source
	// of libration for the moon, ranging up to several degrees.
	variable F = moonArgumentOfLatitude(JD)
	variable Omega = moonMeanLongitudeOfAscendingNode(JD)
	
	// The following are equations 53.1 in Meeus
	variable W = lambdaMinusNutation - Omega
	variable cb = cos(beeta)
	variable sb = sin(beeta)
	variable sw = sin(W)
	variable ci = cos(I)
	variable si = sin(I)
	variable A = atan2(sw*cb*cos(I) - sb*si, cos(W)*cb)
	
	variable lprime = A - F
	
	// Normalize to small angles
	if (lprime > 180) // degrees
		lprime-= 360 // degrees
	elseif (lprime < -180) // degrees
		lprime+= 360 // degrees
	endif
	
	variable bprime = asin(-sw*cb*si - sb*ci)
	
	variable T = meeusT(JD)
	variable K1 = (119.75 + 131.849*T) //degrees
	variable K2 =  (72.56 +  20.186*T) //degrees
	
	variable D = moonMeanElongation(JD)
	variable M = moonCalcSunMeanAnomaly(JD)
	variable Mprime = moonMeanAnomaly(JD)
	variable E = moonCalcEarthEccentricity(JD)
	
	variable rho = -0.02752*cos(Mprime) - 0.02245*sin(F) + 0.00684*cos(Mprime - 2*F) + \
	-0.00293*cos(2*F) - 0.00085*cos(2*F - 2*D) - 0.00054*cos(Mprime - 2*D) + \
	-0.00020*sin(Mprime + F) - 0.00020*cos(Mprime + 2*F) + \
	-0.00020*cos(Mprime - F) + 0.00014*cos(Mprime + 2*F - 2*D) //degree
	
	variable sigma = (-0.02816*sin(Mprime) + 0.02244*cos(F) - 0.00682*sin(Mprime - 2*F) + \
	-0.00279*sin(2*F) - 0.00083*sin(2*F - 2*D) + 0.00069*sin(Mprime - 2*D) + \
	0.00040*cos(Mprime + F) - 0.00025*sin(2*Mprime) + \
	-0.00023*sin(Mprime + 2*F) + 0.00020*cos(Mprime - F) + \
	0.00019*sin(Mprime - F) + 0.00013*sin(Mprime + 2*F - 2*D) + \
	-0.00010*cos(Mprime - 3*F)) //degree
	
	variable tau = (0.02520*E*sin(M) + 0.00473*sin(2*Mprime - 2*F) - 0.00467*sin(Mprime)+ \
	0.00396*sin(K1) + 0.00276*sin(2*Mprime - 2*D) + 0.00196*sin(Omega) + \
	-0.00183*cos(Mprime - F) + 0.00115*sin(Mprime - 2*D) + \
	-0.00096*sin(Mprime - D) + 0.00046*sin(2*F - 2*D) + \
	-0.00039*sin(Mprime - F) - 0.00032*sin(Mprime - M - D) + \
	0.00027*sin(2*Mprime - M - 2*D) + 0.00023*sin(K2) + \
	-0.00014*sin(2*D) + 0.00014*cos(2*Mprime - 2*F) + \
	-0.00012*sin(Mprime -  2*F) - 0.00012*sin(2*Mprime) + \
	0.00011*sin(2*Mprime - 2*M - 2*D)) //degree
	
	variable l2prime = -tau + (rho*cos(A) + sigma*sin(A))*tan(bprime)
	variable b2prime = sigma*cos(A) - rho*sin(A)
	
	l = lprime + l2prime
	b = bprime + b2prime
	
	// Now calculate the position angle P of the moon's axis (chapter 53)
	variable V = Omega + deltaPsi + sigma/si
	variable sir = sin(I + rho)
	variable X = sir*sin(V)
	
	variable epsilon = trueObliquityOfEcliptic(JD)
	variable Y = sir*cos(V)*cos(epsilon) - cos(I + rho)*sin(epsilon)
	omega = atan2(X, Y)
	
	[Variable ra,Variable decl] = moonApparentRADecl(JD)
	P = mod(asin(sqrt(X^2 + Y^2)*cos(ra - omega) / cos(b)), 360)
	
	return [l, b, P]
end
//___________________________________________________________________________________________________
// Calculate precession corrections for the specified time.
//
//    These are the "rigorous method" calculations for precession from Meeus,
//    chapter 21.  The following is equation 21.3.
//
//    arguments:
//    targetEpoch:  The target time to calculate the precession.
//
//    initialEpoch: The initial epoch in which coordinates are specified.
//    Since star positions are usually referenced to the epoch J2000.0, this
//    is given as the default value for initialEpoch.
//
//    returns (zeta, z, theta)
//
Function [Variable zeeta,Variable z, Variable theta] precessionCorrections(Variable dat)
	
	Variable t = dat / 36525 // julian century
	
	zeeta = 0.6406161*t + 0.00008385556*t^2 + 0.000004999444*t^3
	z = 0.6406161*t + 0.0003040778*t^2 + 0.000005056389*t^3
	theta = 0.556753*t - 0.0001185139*t^2 - 0.00001162028*t^3 
	
	return [zeeta, z, theta]
end
//___________________________________________________________________________________________________
// Precesses the coordinates to the target epoch and to the equinox
//    of the target date.  These implement equations 21.4 in Meeus.
//
//    arguments:
//      date:  The target date
//      ra0:   The right ascension referred to the given refEpoch, as an angle
//             or a time.
//      decl0: The declination referred to the given refEpoch
//      refEpoch:  The epoch of the original coordinates.
//         Since star positions are usually referenced to the epoch J2000.0, this
//         is given as the default value for refEpoch.
//
//    returns:
//    (ra, decl) : The precessed right ascension and declination of the
//                 coordinates related to the equinox of date.
//
Function [Variable ra,Variable  decl] precess(Variable JD,Variable ra0,Variable decl0) 
	
	[Variable zeeta,Variable z,Variable theta] = precessionCorrections(JD)
	
	// These are equations 21.4
	Variable cd0 = cos(decl0)
	Variable sd0 = sin(decl0)
	Variable aoc = ra0 + zeeta
	Variable saoc = sin(aoc)
	Variable caoc = cos(aoc)
	Variable ctheta = cos(theta)
	Variable stheta = sin(theta)
	Variable A = cd0*saoc
	Variable B = ctheta*cd0*caoc - stheta*sd0
	Variable C = stheta*cd0*caoc + ctheta*sd0
	Variable raMinusZ = atan2(A, B)
	ra = raMinusZ + z
	decl = asin(C)
	
	return [ra, decl]
end
//___________________________________________________________________________________________________
// Calculates the apparent position of a star to high accuracy using
//    coordinates from a specified equinox.  It corrects for:
//      * Proper motion (assumed to be uniform)
//      * Precession
//      * Nutation
//      * Annual aberration
//
//    It does *NOT* correct for refraction, and these calculations are
//    geocentric, that is, referred to the center of the earth.
//    You will want to use something like refractedAzimuthAltitude to find its
//    refracted azimuth and altitude for a specific spot on the earth.
//
//    See chapter 23 in Meeus for more explanation.
//
//   arguments:
//    date:  The target date for calculation
//    ra0:   The right ascension in some reference epoch (as angle or time)
//    decl0: The declination in some reference epoch (as an angle)
//    raV:   The proper motion in right ascension (as angle or time) per time
//    declV: The proper motion in declination (as angle or time) per time
//    distance:  The distance to the star in some reference epoch (or undef if
//               unknown)
//    radialV:  The radial velocity of the star relative to the solar system
//               (positive means distance increasing) or undef if unknown
//    refEpoch: The reference epoch in which the coordinates are specified.
//      Since star positions are often referenced to the epoch J2000.0, this
//      is given as the default value for initialEpoch.
//
//    distance and radialV are rarely important.  Even a close star like Sirius
//    with high radial velocity gives an error of less than 0.04 arcsecond
//    between 1900 and 2100, but may be necessary when extrapolating thousands
//    of years into the past or future.
//
//    returns:
//    (ra, decl) in the epoch and mean equinox of the date
//
Function [Variable ra,Variable  decl] apparentPosition(Variable JD,Variable  ra0, Variable decl0, Variable raV, Variable declV, Variable distance, Variable radialV)
	Variable refEpoch = DateToJulian(2000,01,01)+0.5 
	
	// Correct ra to angle if necessary
	//   if ra0 conforms time
	//      ra0 = ra0 * (circle/day)
	
	// Correct velocity in right ascension to angle/time
	//   if raV conforms (time/time)
	//      raV = raV * (circle/day)
	
	//   if (distance == undef OR radialV == undef)
	//   
	//      // Correct for proper motion without radial velocity
	//      timeOffset = date - refEpoch
	//      dra = raV * timeOffset
	//      ddecl = declV * timeOffset
	
	//      ra0 = ra0 + dra
	//      decl0 = decl0 + ddecl
	//    else
	
	// Correct for proper motion with known radial velocity and distance
	// Meeus p. 141
	// Since Frink tracks the units of measure, you don't have to go through
	// the convolutions of turning things into parsecs/year, radians/year,
	// etc. 
	Variable x = distance*cos(decl0)*cos(ra0)
	Variable y = distance*cos(decl0)*sin(ra0)
	Variable z = distance*sin(decl0)
	
	Variable dx = (x/distance)*radialV - z*declV*cos(ra0) - y*raV
	Variable dy = (y/distance)*radialV - z*declV*sin(ra0) + x*raV
	Variable dz = (z/distance)*radialV + distance*declV*cos(decl0)
	
	Variable timeOffset = JD - refEpoch
	
	Variable xp = x + timeOffset*dx
	Variable yp = y + timeOffset*dy
	Variable zp = z + timeOffset*dz
	
	ra0 = atan2(yp, xp)
	decl0 = atan2(zp, sqrt(xp^2 + yp^2))
	
	// Now correct for precession.  (ra0, decl0) are now the coordinates for the
	// for the mean equinox of the reference time, but for the target epoch.
	[ra, decl] = precess(JD, ra0, decl0)
	
	// Now correct for nutation.  (See Meeus p. 150).
	[Variable deltaRA1,Variable deltaDecl1] = highAccuracyNutationInRADecl(JD, ra, decl)
	
	// Now correct for annual aberration.  See Meeus eq. 23.2
	[Variable deltaRA2,Variable deltaDecl2] = annualAberration(JD, ra, decl)
	
	ra = ra + deltaRA1 + deltaRA2
	decl = decl + deltaDecl1 + deltaDecl2
	
	return [ra, decl]
end
//___________________________________________________________________________________________________


